//------------------------------------------------------------------
//                                                                  
// Module      : UserEJBBean.java                                   
//                                                                  
// Description : CandleNet eBP JMS Outbound toolkit Sample EJB      
//                                                                  
//------------------------------------------------------------------
//                                                                  
// Copyright Candle Corporation 2001,2003
//                                                                  
//------------------------------------------------------------------
//                                                                  
// Details:                                                         
//    KxoOutbound toolkit Sample User EJB Interface                        
//                                                                  
// Invocation Syntax:                                               
//    String inquire(string)                                                
//    String buy(string)                                                
// Use the UserEJBHome interface to create a UserEJB session bean.  
//                                                                  
//                                                                  
//------------------------------------------------------------------
//                                                                  
// Processing:                                                      
//                                                                  
// The UserEJBBean class is a session Enterprise JavaBean (EJB) that
// is used to represent some middle teir business logic processing. 
// During its processing it wishes to invoke our eBP CEJO smaple    
// Access Bean.                                                     
// In a real system, this would likely be an existing EJB being     
// modified to invoke some eBP business service.                    
//                                                                  
// The deployment descriptor should mark this bean as a stateless   
// session EJB.                                                     
// The transaction attribute for inquire should be TX_NOT_SUPPORTED.            
// The transaction attribute for buy should be TX_REQUIRED.
// The file UserEJBDD.xml is provided with these settings.
//                                                                  
//------------------------------------------------------------------
//                                                                  
// Return Value:                                                    
//  A string containing the parsed return message from the invoked  
//  eBP Business Service or a transaction id for the buy method
//                                                                  
//------------------------------------------------------------------
package com.candle.roma.kxo.outbound.sample;

import com.candle.roma.kxo.LogHelper;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import java.rmi.RemoteException;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
// bean-managed transactions
import javax.transaction.*;


/**
  * The UserEJBBean class implements a sample middle teir of
  * an EJB application which will demonstrate calling a
  * eBP business service via a business method implmented
  * in the sample Access Bean
  * @author     Candle
  * @version 1.0  $Revision: 1.11 $
  */
public class UserEJBBean extends Object implements SessionBean {

  private static final String KxoOutboundSampleCopyright
      = "(c) Copyright Copyright Candle Corporation 2002";

  // =======================================================================
  // kxo-websphere integrated log helper wrapper
  private static final LogHelper LOG = 
      new LogHelper("com.candle.roma.kxo.outbound.sample.UserEJBBean");

  // Support for bean-managed transactions 
  // The SessionContext is required to access the UserTransaction instance.
  // The bean transactional deployment property must be set TX_BEAN_MANAGED
  // for this to be allowed by the application server 
  private boolean dobeanmanagedTX = false;
  private SessionContext sc;


  /**
    Creates an instance of a UserEJB bean.
  */
  public UserEJBBean (){}

  /**
    This is not a part of SessionBean interface; it is defined implicitly
    through the "home" interface.  This method is called after creation.
  */
  public void ejbCreate ()   {
    // nothing needed to be done by me
  }

  /**
    The activate method is called when the instance is activated
    from its "passive" state.
    The instance should acquire any resource that it has released
    earlier in the ejbPassivate() method.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbActivate () throws RemoteException
  {
    // nothing needed to be done by me
  }

  /**
    The passivate method is called before the instance enters the
    "passive" state.
    The instance should release any resources that that it
    can re-acquire later in the ejbActivate() method.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbPassivate () throws RemoteException
  {
    // nothing needed to be done by me
  }

  /**
    A container invokes this method before it ends the life of
    the session object.
    This happens as a result of a client's invoking a remove
    operation, or when a container decides to terminate the session
    object after a timeout.

    @exception RemoteException Thrown if the remote method call fails.
  */
  public void ejbRemove () throws RemoteException
  {
  }

  /**
    Set the associated session context.
    The container calls this method after the instance creation.

    @param ctx An SessionContext interface for the instance.
    @exception RemoteException Thrown if the remote method call fails.
  */
  public void setSessionContext (SessionContext ctx) throws RemoteException
  {
    // if we're not using transactions we don't need the context, but for ...
    // bean-managed transactions
    // Store the SessionContext for use in the business methods.
    this.sc = ctx;
  }


  /**
   ** Method: buy()
   ** This method is called by the sample servlet when it wants
   ** to trigger a buy transaction.  It will do some middle teir
   ** processing and will make the "business-oriented" call to the
   ** eBP business service via the sample Access Bean. That method
   ** will issue a datagram to eBP and provide the msg transaction.
   ** The access bean maps to KxoOutbound EJB eBPpost method which requires
   ** a JTA transaction context, thus this method is marked as
   ** TX_REQUIRED on the deployment descriptor
   **
   ** @return String audit tally tag from post to business service
   ** @param  msg a String identifying the item to buy
   ** @since 1.01
   */
  public String buy(String msg) 
  {
    String   response;
    boolean  validProd   = true;

    // For the sake of the smaple we tag the response with the compoent
    response   = "@UserEJB::buy. ";
    response   += process(msg, true);
    
    return response;
  }

  /**
   ** Method: inquire()
   ** This method s called by the sample servlet when it wants
   ** information.  It will do some middle teir processing and
   ** will make the "business-oriented" call to the eBP business
   ** service via the sample Access Bean. That method will
   ** issue a request and await a response from eBP betfore 
   ** returning to this method - thus the caller is blocked until
   ** the service responds or a timeout occurs.
   ** The access bean maps to KxoOutbound EJB eBPprocess method which 
   ** due to needing to issue a put and a get operation can not work
   ** in a JTA transaction context, thus this method is marked as
   ** TX_NOT_SUPPORTED on the deployment descriptor
   **
   ** @return String response from the business method.
   ** @param  msg a String identifying the inquired item
   ** @since 1.01
   */
  public String inquire(String msg) 
  {
    String   response;

    // For the sake of the smaple we tag the response with the compoent
    response   = "@UserEJB::inquire. ";
    response   += process(msg, false);
    
    return response;
  }

  /**
   ** This method is called by the sample servlet when it wants
   ** to send free form text to a specified business service
   ** The access bean maps to KxoOutbound EJB eBPprocess method which 
   ** due to needing to issue a put and a get operation can not work
   ** in a JTA transaction context, thus this method is marked as
   ** TX_NOT_SUPPORTED on the deployment descriptor
   **
   ** @return String response from the business method.
   ** @param  msg a String identifying the inquired item
   ** @since 1.01
   */
  public String serviceRequest(String srv, String msg) 
  {
    String   request;
    String   response;

    // ------------------------------------------------
    // Middle tier to  eBP Business Service backend link
    // Here we call a eBP Business service via an
    // business interface provided by the sample
    // XML Access Bean
    XmlAccessBean srvAccess = new XmlAccessBean(srv);

    // ------------------------------------------------
    // Wrap the text as XML if not already
    if ( ! msg.regionMatches(0, "<?xml", 0, 5) )
    {
      request = new String("<?xml version=\"1.0\" ?> <msg>" + 
                           msg + "</msg>");
    } else {
      request = msg;
    }

    // ------------------------------------------------
    // Send and see what we get
    try {
      response = srvAccess.requestXmlResponse(request);
    } catch (Exception ex) {
      response = new String("Failed: " + ex.toString());
    }

    // -------------------------------------------------
    // If request was not XML we should unpack it

    return response;
  }



  //------------------------------------------------------------------------
  /**
   ** Method: process()
   **
   ** @param  msg String identifying the inquired item
   ** @param  datagram Boolean set true for datagram post and false for
   **         synchronous inquiry
   ** @return String response from the business method.
   ** @exception RemoteException Thrown if the remote method call fails
   ** @since 1.01
   */
  private String process(String msg, boolean datagram)
    throws EJBException
  {
    String   response = "";
    boolean  validProd   = true;
    UserTransaction ut   = null;


    if (dobeanmanagedTX)
    {
      try {
        // For Bean managed TX Obtain the UserTransaction instance
        ut = sc.getUserTransaction();
        // Begin a new Unit Of Transactional Work
        ut.begin();
      } catch ( javax.transaction.NotSupportedException ex ) {
       LOG.error("UserBean caught javax.transaction.NotSupported Exception", ex);
       response += "Error: transaction NotSupported";
      } catch ( Exception ex ) {
       LOG.error("Error UserBean caught: Exception", ex);
       response += "Error: begin transaction Exception : " + ex.toString();
      }
    }

    // ---------------------------------------------------------
    // Middle Teir Business Logic
    // a real application would really validate the supplied productId
    // and populate a real description at this point. 
    // For our simple example, we just hard-code them here in place of
    // the middle teir logic
    int            productId   = 0;
    StringBuffer   description = new StringBuffer();
    StringBuffer   information = new StringBuffer();

    try {
      Integer val = new Integer(msg);
      productId = val.intValue();
    }
    catch (Exception e) {
      //
      // Exception occured - not passed a valid product code
      //
      response += "Error: Invalid product code passed. ] ";
      return response;
    }

    switch ( productId )
    {
    case 0 :
      description.append("No product");
      validProd = false;
      break;
    case 11111 :
      description.append("Default product");
      break;
    case 12345 :
      description.append("eBP enabled widget");
      break;
    case 99999 :
      response += "Please Select product code. ]";
      validProd = false;
      break;
    default :
      description.append("Product is a " + msg + " widget");
      break;
    }
      
    if ( validProd )
    {
      // ------------------------------------------------
      // Middle tier to  eBP Business Service backend link
      // Here we call a eBP Business service via an
      // business interface provided by the sample
      // KxoOutbound Access Bean
      AccessBean plumbing = new AccessBean();

      // -- Post or Inquire
      if ( datagram ) {
        // ------------------------------------------------
        // Post a transactional buy order
        String tallyCode = plumbing.buyProductByXml(productId,
                                          description,
                                          information);
        // Compose the response string
        response += "<DL>";
        response += "<DT>Requested Product<DD>" + description;
        response += "<DT>Product purchase response<DD>";
        response += tallyCode + information;
        response += "</DL>";
      } else {
        // -------------------------------------------------
        // Issue a none tranactional inquiry
        String cost = plumbing.getProductDetailsByXml(productId,
                                                 description,
                                                 information);
        // Compose the response string
        response += "<DL>";
        response += "<DT>Requested Product<DD>" + description;
        response += "<DT>Product cost retreived from eBP Service<DD>";
        response += cost;
        response += "<DT>Product information retreived from eBP Service<DD>";
        response += information;
        response += "</DL>";
      }
    }

    if (dobeanmanagedTX)
    {
      // complete the transactional unit of work
      try {
        ut.commit();
      } catch ( javax.transaction.RollbackException ex ) {
       LOG.debug("Error UserBean caught: javax.transaction.RollbackException", ex);
       response += "Error: transaction Rolledback";
      } catch ( Exception ex ) {
       LOG.debug("Error UserBean caught: Exception", ex);
       response += "Error: commit transaction Exception";
      }
    }
    return response;
  }
}
