//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcobea.cpp,v 1.14 2003/12/11 18:30:03 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcobea.cpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The RomaBusinessElement public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoicxt.hpp>
#include <kxcoipxy.hpp>
#include <kxcobe.hpp>

TRACE_MODULE_VERSION($Revision: 1.14 $);

//--------------------------------------------------------------------
// Define RTTI for the class.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaBusinessElement);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaBusinessElement::RomaBusinessElement()
{
    TRACE_ENTRY(RomaBusinessElement::RomaBusinessElement, << TRACE(this));

    m_impl = new RomaImplElement;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Establishes connection.
//--------------------------------------------------------------------
RomaBusinessElement::RomaBusinessElement(const char* name,
                                         KXCULONG options,
                                         const RomaSecurityDef& secDef,
                                         const char* model,
                                         const RomaILSData& ilsData,
                                         const RomaDirectoryAccess& auth)
{
    TRACE_ENTRY(RomaBusinessElement::RomaBusinessElement, << TRACE(this));

    m_impl = new RomaImplElement;
    connect(name, options, secDef, model, ilsData, auth);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
// Terminates connection.
//--------------------------------------------------------------------
RomaBusinessElement::~RomaBusinessElement()
{
    TRACE_ENTRY(RomaBusinessElement::~RomaBusinessElement, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Establish connection.
//--------------------------------------------------------------------
bool RomaBusinessElement::connect(const char* name,
                                  KXCULONG options,
                                  const RomaSecurityDef& secDef,
                                  const char* model,
                                  const RomaILSData& ilsData,
                                  const RomaDirectoryAccess& auth)
{
    TRACE_ENTRY(RomaBusinessElement::connect, << TRACE(this));

    bool success = m_impl->connect(KXCOCALLTYPE_BECONNECT, 
        name, options, secDef, model, ilsData, auth);
    m_impl->setTermIndicator(false);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Put the message.
//--------------------------------------------------------------------
bool RomaBusinessElement::put(RomaBeResponse& response)
{
    return m_impl->put(response);
}

//--------------------------------------------------------------------
// Get a message.
// Message data buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaBeRequest* RomaBusinessElement::get()
{
    RomaGetOptions options;
    return m_impl->get(*this, options);
}

//--------------------------------------------------------------------
// Get a message.
// Message data buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaBeRequest* RomaBusinessElement::get(RomaGetOptions& options)
{
    return m_impl->get(*this, options);
}

//--------------------------------------------------------------------
// Get a message.
// Message data will be placed into the specified buffers.
//--------------------------------------------------------------------
RomaBeRequest* RomaBusinessElement::get(RomaGetOptions& options,
                                        void* msgBuffer,
                                        int&  msgLength,
                                        void* tagBuffer,
                                        int&  tagLength)
{
    return m_impl->get(*this, options, msgBuffer, msgLength, 
        tagBuffer, tagLength);
}

//--------------------------------------------------------------------
// Unmarshal an incoming message.
// This method may be overriden by an application to perform message
// unmarshaling. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaBusinessElement::createRequest(const RomaBeRequest& baseRequest,
                                        RomaBeRequest*& newRequest)
{
    TRACE_ENTRY(RomaBusinessElement::createRequest, << TRACE(this));

    // Fast copy, just the implementation pointer will be shared.
    newRequest = new RomaBeRequest(baseRequest);

    TRACE_INT_RETURN(newRequest != NULL);
}

//--------------------------------------------------------------------
// Discover the termination indicator state.
//--------------------------------------------------------------------
bool RomaBusinessElement::isTermIndicator() const
{
    return m_impl->getTermIndicator();
}

//--------------------------------------------------------------------
// Terminate connection.
//--------------------------------------------------------------------
bool RomaBusinessElement::disconnect()
{
    return m_impl->disconnect();
}

//--------------------------------------------------------------------
// Indicates whether the object is connected.
//--------------------------------------------------------------------
bool RomaBusinessElement::isConnected() const
{
    return m_impl->isConnected();
}

//--------------------------------------------------------------------
// Commit a local transaction.
//--------------------------------------------------------------------
bool RomaBusinessElement::commitLocal()
{
    return m_impl->commitLocal();
}

//--------------------------------------------------------------------
// Rollback a local transaction.
//--------------------------------------------------------------------
bool RomaBusinessElement::rollbackLocal()
{
    return m_impl->rollbackLocal();
}

//--------------------------------------------------------------------
// Close the theme.
//--------------------------------------------------------------------
bool RomaBusinessElement::closeTheme(const RomaTheme& theme)
{
    return m_impl->closeTheme(theme);
}

//--------------------------------------------------------------------
// Delete the named storage queue.
//--------------------------------------------------------------------
bool RomaBusinessElement::deleteStorageQueue(const char* queueName)
{
    return m_impl->deleteStorageQueue(queueName);
}

//--------------------------------------------------------------------
// Put the exception message.
//--------------------------------------------------------------------
bool RomaBusinessElement::putException(RomaExceptionMessage& message,
                                       const RomaExceptionOptions& options)
{
    return m_impl->putException(message, options);
}

//--------------------------------------------------------------------
// Report the error to the EAR.
//--------------------------------------------------------------------
bool RomaBusinessElement::reportError(RomaExceptionMessage& message,
                                      KXCULONG& carriedOutActions,
                                      const RomaReportOptions& options)
{
    return m_impl->reportError(message, carriedOutActions, options);
}

//--------------------------------------------------------------------
// Send the alert.
//--------------------------------------------------------------------
bool RomaBusinessElement::sendAlert(const RomaLogInformation& logInfo)
{
    return m_impl->sendAlert(logInfo);
}

//--------------------------------------------------------------------
// Get ILS data provided on initialization.
//--------------------------------------------------------------------
RomaILSData* RomaBusinessElement::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Indicates whether the last error information is present.
//--------------------------------------------------------------------
bool RomaBusinessElement::anyError() const
{
    return m_impl->anyError();
}

//--------------------------------------------------------------------
// Get the last error information.
//--------------------------------------------------------------------
RomaError& RomaBusinessElement::error()
{
    return m_impl->getError();
}

//--------------------------------------------------------------------
// Delete the last error information.
//--------------------------------------------------------------------
void RomaBusinessElement::resetError()
{
    m_impl->resetError();
}

//--------------------------------------------------------------------
// Write information to the user log.
//--------------------------------------------------------------------
bool RomaBusinessElement::writeLog(const RomaLogInformation& logInfo)
{
    return m_impl->writeLog(logInfo);
}

//--------------------------------------------------------------------
// Write information to the system log.
//--------------------------------------------------------------------
bool RomaBusinessElement::writeSysLog(const RomaLogInformation& logInfo)
{
    return m_impl->writeSysLog(logInfo);
}

//--------------------------------------------------------------------
// Inserts the business service into the workflow of the theme.
//--------------------------------------------------------------------
bool RomaBusinessElement::insertWorkflow(RomaTheme& newTheme,
                                         const RomaTheme& curTheme,
                                         const RomaBusinessService& bs,
                                         bool close) const
{
    return m_impl->insertWorkflow(newTheme, curTheme, bs, close, false);
}

//--------------------------------------------------------------------
// Replaces the workflow of the theme with a workflow based on the
// specified business service.
//--------------------------------------------------------------------
bool RomaBusinessElement::replaceWorkflow(RomaTheme& newTheme,
                                          const RomaTheme& curTheme,
                                          const RomaBusinessService& bs,
                                          bool close) const
{
    return m_impl->insertWorkflow(newTheme, curTheme, bs, close, true);
}

//--------------------------------------------------------------------
// Sets the business flow information on the context.
//--------------------------------------------------------------------
bool RomaBusinessElement::setBusinessFlow(
    const RomaBusinessFlow& bf) const
{
    return m_impl->setBusinessFlow(bf);
}

//--------------------------------------------------------------------
// Creates a new theme that is copy of the specified theme.
//--------------------------------------------------------------------
bool RomaBusinessElement::copyTheme(RomaTheme& newTheme,
                                    const RomaTheme& curTheme) const
{
    return m_impl->copyTheme(newTheme, curTheme);
}

//--------------------------------------------------------------------
// Merges ETE information into the theme.
//--------------------------------------------------------------------
bool RomaBusinessElement::mergeEteInfo(RomaTheme& newTheme,
                                       const RomaTheme& curTheme,
                                       const RomaEteInfo& ete,
                                       bool close) const
{
    return m_impl->mergeEteInfo(newTheme, curTheme, ete, close);
}

//--------------------------------------------------------------------
// Extracts theme information.
//--------------------------------------------------------------------
bool RomaBusinessElement::queryTheme(RomaThemeInfo& info,
                                     const RomaTheme& theme) const
{
    return m_impl->queryTheme(info, theme, false);
}

//--------------------------------------------------------------------
// Creates a new theme from the specified theme information.
//--------------------------------------------------------------------
bool RomaBusinessElement::buildTheme(RomaTheme& newTheme,
                                     const RomaTheme& curTheme,
                                     const RomaThemeInfo& info,
                                     unsigned options,
                                     bool close) const
{
    return m_impl->buildTheme(newTheme, curTheme, info, options,
        close, false);
}

//--------------------------------------------------------------------
// Extracts message information.
//--------------------------------------------------------------------
bool RomaBusinessElement::interpretMessage(RomaMessageInfo& info,
                                           const RomaBeRequest& msg,
                                           unsigned options) const
{
    return m_impl->interpretMessage(
        info, RomaImplProxy::getImpl(msg), options);
}

//--------------------------------------------------------------------
// Operator casting the object to the context handle. Deprecated.
//--------------------------------------------------------------------
HKXC RomaBusinessElement::operator *() const
{
    return (HKXC)(*m_impl);
}

//--------------------------------------------------------------------
// Operator casting the object to the context handle.
//--------------------------------------------------------------------
RomaBusinessElement::operator HKXC() const
{
    return (HKXC)(*m_impl);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaBusinessElement::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaBusinessElement, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

