//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcobrsa.cpp,v 1.10 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcobrsa.cpp,v $
// $Revision: 1.10 $
//--------------------------------------------------------------------
// The RomaBeResponse public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcobrs.hpp>
#include <kxcodefs.hpp>

TRACE_MODULE_VERSION($Revision: 1.10 $);

//--------------------------------------------------------------------
// Define RTTI for the class.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaBeResponse);

//--------------------------------------------------------------------
// Default constructor.
// Optionally accepts a theme to associate the message with it.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(const RomaTheme& theme)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(theme));

    m_impl = new RomaImplElementResponse(theme);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the message with the theme and data.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(const RomaTheme& theme,
                               const void* msgBuffer,
                               int         msgLength)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(theme)
                                                << TRACE(msgBuffer)
                                                << TRACE(msgLength));

    m_impl = new RomaImplElementResponse(theme);
    m_impl->setMsgBuffer(msgBuffer, msgLength);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the message with the theme, data and tag.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(const RomaTheme& theme,
                               const void* msgBuffer,
                               int         msgLength,
                               const void* tagBuffer,
                               int         tagLength)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(theme)
                                                << TRACE(msgBuffer)
                                                << TRACE(msgLength)
                                                << TRACE(tagBuffer)
                                                << TRACE(tagLength));

    m_impl = new RomaImplElementResponse(theme);
    m_impl->setMsgBuffer(msgBuffer, msgLength);
    m_impl->setTagBuffer(tagBuffer, tagLength);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the message with the theme, the message ID and
// the correlation ID of the specified request.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(const RomaBeRequest* request)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(request));

    m_impl = new RomaImplElementResponse;

    if (request)
    {
        m_impl->setTheme(request->theme());
        m_impl->setMsgId(request->msgId());
        m_impl->setCorrelId(request->correlId());
    }
    else
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the specified internal implementation
// instance.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(RomaImplElementResponse* impl) 
    : m_impl(impl)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(impl));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaBeResponse::~RomaBeResponse()
{
    TRACE_ENTRY(RomaBeResponse::~RomaBeResponse, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
// It creates the shallow copy of the request. The original response
// and its copy share the same internal implementation instance.
//--------------------------------------------------------------------
RomaBeResponse::RomaBeResponse(const RomaBeResponse& resp)
    : m_impl(resp.m_impl)
{
    TRACE_ENTRY(RomaBeResponse::RomaBeResponse, << TRACE(this)
                                                << TRACE(resp));

    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
// It returns the shallow copy of the request. The original response
// and its copy share the same internal implementation instance.
//--------------------------------------------------------------------
RomaBeResponse& RomaBeResponse::operator = (const RomaBeResponse& resp)
{
    TRACE_ENTRY(RomaBeResponse::operator =, << TRACE(this)
                                            << TRACE(resp));

    if (m_impl != resp.m_impl)
    {
        m_impl->releaseInstance();
        m_impl = resp.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// This method is called right before message sending to perform
// data marshaling. The default implementation does nothing.
//--------------------------------------------------------------------
bool RomaBeResponse::marshalResponse()
{
    return true;
}

//--------------------------------------------------------------------
// Indicates whether the theme should be closed.
//--------------------------------------------------------------------
void RomaBeResponse::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Indicates whether the theme should be retained.
//--------------------------------------------------------------------
void RomaBeResponse::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Indicates that the message should be sent as a notification.
//--------------------------------------------------------------------
void RomaBeResponse::setNotification(bool f)
{
    m_impl->setNotification(f);
}

//--------------------------------------------------------------------
// Indicates whether notification messages are allowed.
//--------------------------------------------------------------------
void RomaBeResponse::setNoNotification(bool f)
{
    m_impl->setNoNotification(f);
}

//--------------------------------------------------------------------
// Set a securty definition.
//--------------------------------------------------------------------
void RomaBeResponse::setSecurityDef(const RomaSecurityDef& secDef)
{
    m_impl->setSecurityDef(secDef);
}

//--------------------------------------------------------------------
// Get the theme associated with the message.
//--------------------------------------------------------------------
const RomaTheme& RomaBeResponse::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Associates the message with the theme.
//--------------------------------------------------------------------
void RomaBeResponse::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Get the message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaBeResponse::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Set the message ID.
//--------------------------------------------------------------------
void RomaBeResponse::setMsgId(const RomaMsgId& msgId)
{
    m_impl->setMsgId(msgId);
}

//--------------------------------------------------------------------
// Get the correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaBeResponse::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Set the correlation ID.
//--------------------------------------------------------------------
void RomaBeResponse::setCorrelId(const RomaMsgId& correlId)
{
    m_impl->setCorrelId(correlId);
}

//--------------------------------------------------------------------
// Set the message priority.
//--------------------------------------------------------------------
void RomaBeResponse::setPriority(int priority)
{
    m_impl->setPriority(priority);
}

//--------------------------------------------------------------------
// Sets the message persistence.
//--------------------------------------------------------------------
void RomaBeResponse::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Sets the message format.
//--------------------------------------------------------------------
void RomaBeResponse::setFormat(int format)
{
    m_impl->setFormat(format);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaBeResponse::setMsgName(const char* name)
{
    m_impl->setMsgName(name);
}

//--------------------------------------------------------------------
// Set the message feedback.
//--------------------------------------------------------------------
void RomaBeResponse::setFeedback(int feedback)
{
    m_impl->setFeedback(feedback);
}

//--------------------------------------------------------------------
// Set the message expiry.
//--------------------------------------------------------------------
void RomaBeResponse::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the message code page.
//--------------------------------------------------------------------
void RomaBeResponse::setCodepage(int codepage)
{
    m_impl->setCodepage(codepage);
}

//--------------------------------------------------------------------
// Set the message encoding.
//--------------------------------------------------------------------
void RomaBeResponse::setEncoding(int encoding)
{
    m_impl->setEncoding(encoding);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaBeResponse::setFormatName(const char* name)
{
    m_impl->setFormatName(name);
}

//--------------------------------------------------------------------
// Set message data.
//--------------------------------------------------------------------
bool RomaBeResponse::setMsgBuffer(const void* buffer, int length)
{
    return m_impl->setMsgBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Set tag data.
//--------------------------------------------------------------------
bool RomaBeResponse::setTagBuffer(const void* buffer, int length)
{
    return m_impl->setTagBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Sets the put selector.
//--------------------------------------------------------------------
void RomaBeResponse::setSelector(Roma::PutSelector selector)
{
    m_impl->setSelector(selector);
}

//--------------------------------------------------------------------
// Indicates whether it is allowed to put message under a transaction
// to a non-transactional queue.
//--------------------------------------------------------------------
void RomaBeResponse::setNoSyncAllowed(bool flag)
{
    m_impl->setNoSyncAllowed(flag);
}

//--------------------------------------------------------------------
// Set an API profile name.
//--------------------------------------------------------------------
void RomaBeResponse::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set an error profile name.
//--------------------------------------------------------------------
void RomaBeResponse::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Indicates whether an EMS may be invoked on an error.
//--------------------------------------------------------------------
void RomaBeResponse::setNoEms(bool flag)
{
    m_impl->setNoEms(flag);
}

//--------------------------------------------------------------------
// Indicates whether the message may be transformed during delivery.
//--------------------------------------------------------------------
void RomaBeResponse::setNoTransformation(bool flag)
{
    m_impl->setNoTransformation(flag);
}

//--------------------------------------------------------------------
// Indicates whether the application is ready to be terminated.
//--------------------------------------------------------------------
void RomaBeResponse::setTermSupported(bool flag)
{
    m_impl->setTermSupported(flag);
}

//--------------------------------------------------------------------
// Indicates whether the error profile should be used.
//--------------------------------------------------------------------
void RomaBeResponse::setSuppressErrorProfile(bool flag)
{
    m_impl->setSuppressErrorProfile(flag);
}

//--------------------------------------------------------------------
// Indicates whether Roma should increment the message retry count.
//--------------------------------------------------------------------
void RomaBeResponse::setIncrementRetryCount(bool flag)
{
    m_impl->setIncrementRetryCount(flag);
}

//--------------------------------------------------------------------
// Set ILS data.
//--------------------------------------------------------------------
void RomaBeResponse::setILSData(const RomaILSData& ILSDataSet)
{
    m_impl->setILSData(ILSDataSet);
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaBeResponse::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Set the syncpoint definition.
//--------------------------------------------------------------------
void RomaBeResponse::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaBeResponse::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaBeResponse, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

