//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2001-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcodac.cpp,v 1.5 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcodac.cpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------
// The RomaDirectoryAccess public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoigdt.hpp>
#include <kxcodac.hpp>

TRACE_MODULE_VERSION($Revision: 1.5 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaDirectoryAccess::RomaDirectoryAccess()
{ 
    m_impl = new RomaImplDirAccess;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the user ID and password.
//--------------------------------------------------------------------
RomaDirectoryAccess::RomaDirectoryAccess(const char* id, 
                                         const char* pwd)
{ 
    m_impl = new RomaImplDirAccess;
    m_impl->setUserId(id);
    m_impl->setPassword(pwd);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaDirectoryAccess::~RomaDirectoryAccess()
{
    delete m_impl;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaDirectoryAccess::RomaDirectoryAccess(
    const RomaDirectoryAccess& other)
{ 
    m_impl = new RomaImplDirAccess(*other.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaDirectoryAccess&
RomaDirectoryAccess::operator = (const RomaDirectoryAccess& other)
{ 
    if (m_impl != other.m_impl)
    {
        *m_impl = *other.m_impl;
    }

    return *this;
}

//--------------------------------------------------------------------
// Set the user ID.
//--------------------------------------------------------------------
bool RomaDirectoryAccess::setUserId(const char* id)
{
    return m_impl->setUserId(id);
}

//--------------------------------------------------------------------
// Get the user ID.
//--------------------------------------------------------------------
const char* RomaDirectoryAccess::getUserId() const
{
    return m_impl->getUserId();
}

//--------------------------------------------------------------------
// Set the password.
//--------------------------------------------------------------------
bool RomaDirectoryAccess::setPassword(const char* pwd)
{
    return m_impl->setPassword(pwd);
}

//--------------------------------------------------------------------
// Get the password.
//--------------------------------------------------------------------
const char* RomaDirectoryAccess::getPassword() const
{
    return m_impl->getPassword();
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaDirectoryAccess::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaDirectoryAccess, << TRACE(m_impl)) 
                                          << "->"
                                          << TRACE(*m_impl);
}

