//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoiopt.cpp,v 1.15 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoiopt.cpp,v $
// $Revision: 1.15 $
//--------------------------------------------------------------------
// The internal implementation of the various Roma options objects.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoiopt.hpp"
#include "kxcoipxy.hpp"

TRACE_MODULE_VERSION($Revision: 1.15 $);


//====================================================================
// The RomaImplOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
// Optionally initializes the object with the specified options.
//--------------------------------------------------------------------
RomaImplOptions::RomaImplOptions(KXCULONG opt) 
    : m_ilsOutgoing(NULL_HKXCILS), m_options(opt), m_status(0)
{
    *m_apiProfile = *m_errorProfile = 0;
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaImplOptions::setErrorProfile(const char* name)
{
    copyZString(m_errorProfile, name, sizeof(m_errorProfile));
}

//--------------------------------------------------------------------
// Set the API profile name.
//--------------------------------------------------------------------
void RomaImplOptions::setApiProfile(const char* name)
{
    copyZString(m_apiProfile, name, sizeof(m_apiProfile));
}

//--------------------------------------------------------------------
// Set ILS data.
//--------------------------------------------------------------------
void RomaImplOptions::setILSData(const RomaILSData& data)
{
    m_ilsOutgoing = (HKXCILS)RomaImplProxy::getImpl(data);
}

//--------------------------------------------------------------------
// Set the message theme.
//--------------------------------------------------------------------
void RomaImplOptions::setTheme(HKXCTHEME theme)
{
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplOptions::trace(RomaTraceStream& s) const
{
    s << TRACE((char*)m_apiProfile) << TRACE((char*)m_errorProfile)
      << TRACE_HEX(m_options) << TRACE(m_status) << TRACE(m_ilsOutgoing)
      << TRACE(m_theme) << TRACE(m_syncpoint);
}


//====================================================================
//  The RomaImplCommonGetOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
// Optionally initializes the object with the specified options.
//--------------------------------------------------------------------
RomaImplCommonGetOptions::RomaImplCommonGetOptions(KXCULONG opt) 
    : RomaImplOptions(opt), 
      m_waitTimeout(KXCGOWAIT_USEDEFAULT), 
      m_subjectStyle(KXCSUBJECTSTYLE_NONE)
{
}

//--------------------------------------------------------------------
// Set get options.
//--------------------------------------------------------------------

void RomaImplCommonGetOptions::setAutoAlloc(bool b)
{
    if (b) m_options |= KXCGOPT_AUTO_ALLOC;
    else   m_options &= ~KXCGOPT_AUTO_ALLOC;
}

void RomaImplCommonGetOptions::setNoEms(bool b)
{
    if (b) m_options |= KXCGOPT_NOEMS;
    else   m_options &= ~KXCGOPT_NOEMS;
}

void RomaImplCommonGetOptions::setNoNotification(bool b)
{
    if (b) m_options |= KXCGOPT_NONOTIFY;
    else   m_options &= ~KXCGOPT_NONOTIFY;
}

void RomaImplCommonGetOptions::setAllowNative(bool b)
{
    if (b) m_options |= KXCGOPT_ALLOWNATIVE;
    else   m_options &= ~KXCGOPT_ALLOWNATIVE;
}

void RomaImplCommonGetOptions::useSignal(bool b)
{
    if (b) m_options |= KXCGOPT_USESIGNAL;
    else   m_options &= ~KXCGOPT_USESIGNAL;
}

void RomaImplCommonGetOptions::setTermSupported(bool b)
{
    if (b) m_options |= KXCGOPT_ALLOWTERMINATE;
    else   m_options &= ~KXCGOPT_ALLOWTERMINATE;
}

void RomaImplCommonGetOptions::setSuppressErrorProfile(bool b)
{
    if (b) m_options |= KXCGOPT_NOERRORPROF;
    else   m_options &= ~KXCGOPT_NOERRORPROF;
}

void RomaImplCommonGetOptions::setCloseTheme(bool b)
{
    m_options &= ~(KXCGOPT_RETAINTHEME | KXCGOPT_CLOSETHEME);
    m_options |= b ? KXCGOPT_CLOSETHEME : KXCGOPT_RETAINTHEME;
}

void RomaImplCommonGetOptions::setRetainTheme(bool b)
{
    setCloseTheme(!b);
}

void RomaImplCommonGetOptions::setUseTheme(bool b)
{
    m_options &= ~(KXCGOPT_ANYTHEME | KXCGOPT_USETHEME);
    m_options |= b ? KXCGOPT_USETHEME : KXCGOPT_ANYTHEME;
}

void RomaImplCommonGetOptions::setAnyTheme(bool b)
{
    setUseTheme(!b);
}

void RomaImplCommonGetOptions::setWaitIndefinite(bool b)
{
    setWaitInterval(b ? KXCGOWAIT_INDEFINITE : 0);
}

//--------------------------------------------------------------------
// Set the operation type.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::setOperation(Roma::Operation op)
{
    m_options &= ~(Roma::OPERATION_MASK);
    m_options |= (KXCULONG)op;
}

//--------------------------------------------------------------------
// Set the get selector.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::setSelector(Roma::GetSelector sel)
{
    m_options &= ~(Roma::SELECTOR_MASK);
    m_options |= (KXCULONG)sel;
}

//--------------------------------------------------------------------
// Set the theme control.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::setThemeControl(Roma::ThemeControl v)
{
    m_options &= ~(Roma::THEME_RETAIN | Roma::THEME_CLOSE);
    m_options |= (KXCULONG)v;
}

//--------------------------------------------------------------------
// Fill the C API structures with options data.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::populate(KXCGETOPT& go, 
                                        KXCSYNC& sync, 
                                        HKXCTHEME& theme) const
{
    memcpy(go.APIProfile, m_apiProfile, sizeof(go.APIProfile));
    memcpy(go.ErrorProfile, m_errorProfile, sizeof(go.ErrorProfile));

    go.Status = m_status;
    go.ReceiveOptions = m_options;
    go.WaitLen = m_waitTimeout;
    go.SubjectStyle = m_subjectStyle;
    kxcSetGetILSData(&go, m_ilsOutgoing);

    if ((m_options & KXCGOPT_USETHEME) == 0)
    {
        go.ReceiveOptions |= KXCGOPT_ANYTHEME;
    }

    if (m_waitTimeout)
    {
        go.ReceiveOptions |= KXCGOPT_WAIT;
    }

    RomaImplProxy::getImpl(m_msgId).copy(go.MsgId);
    RomaImplProxy::getImpl(m_correlId).copy(go.CorrelId);
    RomaImplProxy::getImpl(m_syncpoint).populate(sync); 
    theme = *m_theme;
}

//--------------------------------------------------------------------
// Set options data from the C API structures.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::capture(const KXCGETOPT& go,
                                       HKXCTHEME theme)
{
    m_status = go.Status;
    m_subjectStyle = go.SubjectStyle;
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplCommonGetOptions::trace(RomaTraceStream& s) const
{
    RomaImplOptions::trace(s);
    s << TRACE(m_waitTimeout) << TRACE(m_subjectStyle)
      << TRACE(m_msgId) << TRACE(m_correlId);
}


//====================================================================
// The RomaImplCommonPutOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplCommonPutOptions::RomaImplCommonPutOptions() 
{
}

//--------------------------------------------------------------------
// Fill the C API structures with options data.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::populate(KXCPUTOPT& po, 
                                        KXCSYNC& sync, 
                                        HKXCTHEME& theme) const
{
    memcpy(po.APIProfile, m_apiProfile, sizeof(po.APIProfile));
    memcpy(po.ErrorProfile, m_errorProfile, sizeof(po.ErrorProfile));

    po.RequestOptions = m_options;
    po.Status = m_status;
    kxcSetPutILSData(&po, m_ilsOutgoing);

    RomaImplProxy::getImpl(m_syncpoint).populate(sync); 
    theme = (HKXCTHEME)m_theme;
}

//--------------------------------------------------------------------
// Set options data from the C API structures.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::capture(const KXCPUTOPT& po, HKXCTHEME theme)
{
    m_status = po.Status;
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::trace(RomaTraceStream& s) const
{
    RomaImplOptions::trace(s);
}

//--------------------------------------------------------------------
// Set the put selector.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::setSelector(Roma::PutSelector sel)
{
    m_options &= ~(Roma::USE_MSGID | Roma::USE_CORRELID);
    m_options |= sel & (Roma::USE_MSGID | Roma::USE_CORRELID);
}

//--------------------------------------------------------------------
// Set the put options.
//--------------------------------------------------------------------

void RomaImplCommonPutOptions::setNoSyncAllowed(bool b)
{
    if (b) m_options |= KXCPOPT_ALLOWNOSYNC;
    else   m_options &= ~KXCPOPT_ALLOWNOSYNC;
}

void RomaImplCommonPutOptions::setNoEms(bool b)
{
    if (b) m_options |= KXCPOPT_NOEMS;
    else   m_options &= ~KXCPOPT_NOEMS;
}

void RomaImplCommonPutOptions::setNoNotification(bool b)
{
    if (b) m_options |= KXCPOPT_NONOTIFY;
    else   m_options &= ~KXCPOPT_NONOTIFY;
}

void RomaImplCommonPutOptions::setNotification(bool b)
{
    if (b) m_options |= KXCPOPT_NOTIFY;
    else   m_options &= ~KXCPOPT_NOTIFY;
}

void RomaImplCommonPutOptions::setNoTransformation(bool b)
{
    if (b) m_options |= KXCPOPT_NOXFORM;
    else   m_options &= ~KXCPOPT_NOXFORM;
}

void RomaImplCommonPutOptions::setTermSupported(bool b)
{
    if (b) m_options |= KXCPOPT_ALLOWTERMINATE;
    else   m_options &= ~KXCPOPT_ALLOWTERMINATE;
}

void RomaImplCommonPutOptions::setSuppressErrorProfile(bool b)
{
    if (b) m_options |= KXCPOPT_NOERRORPROF;
    else   m_options &= ~KXCPOPT_NOERRORPROF;
}

void RomaImplCommonPutOptions::setFreeBuffer(bool b)
{
    if (b) m_options |= KXCPOPT_FREEBUFFER;
    else   m_options &= ~KXCPOPT_FREEBUFFER;
}

void RomaImplCommonPutOptions::setNewTheme(bool b)
{
    m_options &= ~(KXCPOPT_USETHEME | KXCPOPT_NEWTHEME);
    m_options |= b ? KXCPOPT_NEWTHEME : KXCPOPT_USETHEME;
}

void RomaImplCommonPutOptions::setUseTheme(bool b)
{
    setNewTheme(!b);
}

void RomaImplCommonPutOptions::setCloseTheme(bool b)
{
    m_options &= ~(KXCPOPT_RETAINTHEME | KXCPOPT_CLOSETHEME);
    m_options |= b ? KXCPOPT_CLOSETHEME : KXCPOPT_RETAINTHEME;
}

void RomaImplCommonPutOptions::setRetainTheme(bool b)
{
    setCloseTheme(!b);
}

bool RomaImplCommonPutOptions::isThemeRetained() const
{
    return ((m_options & KXCPOPT_CLOSETHEME) == 0);
}

//--------------------------------------------------------------------
// Specify true to send a message as a datagram.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::setDatagram(bool b)
{
    if (b) m_options |= KXCPOPT_DATAGRAM;
    else   m_options &= ~KXCPOPT_DATAGRAM ;
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of delivery" flag.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::setCOD(bool b)
{
    if (b) m_options |= KXCPOPT_COD;
    else   m_options &= ~KXCPOPT_COD;
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of arrival" flag.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::setCOA(bool b)
{
    if (b) m_options |= KXCPOPT_COA;
    else   m_options &= ~KXCPOPT_COA;
}

//--------------------------------------------------------------------
// Specify true to increment retry count.
//--------------------------------------------------------------------
void RomaImplCommonPutOptions::setIncrementRetryCount(bool b)
{
    if (b) m_options |= KXCPOPT_INCRETRYCOUNT;
    else   m_options &= ~KXCPOPT_INCRETRYCOUNT;
}


//====================================================================
// The RomaImplExceptionOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplExceptionOptions::RomaImplExceptionOptions() 
{
    *m_service = '\0';
}

//--------------------------------------------------------------------
// Set the business service name.
//--------------------------------------------------------------------
void RomaImplExceptionOptions::setServiceName(const char* name)
{
    copyZString(m_service, name, sizeof(m_service));
}

//--------------------------------------------------------------------
// Get the business service name.
//--------------------------------------------------------------------
const char* RomaImplExceptionOptions::getServiceName() const 
{ 
    return (*m_service ? (const char*)m_service : NULL); 
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplExceptionOptions::trace(RomaTraceStream& s) const
{
    RomaImplCommonPutOptions::trace(s);
    s << TRACE(m_service);
}


//====================================================================
// The RomaImplRetrieveOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplRetrieveOptions::RomaImplRetrieveOptions() 
    : RomaImplCommonGetOptions(
        KXCGOPT_AUTO_ALLOC | KXCGOPT_ANYTHEME | KXCGOPT_RETAINTHEME)
{
}


//====================================================================
// The RomaImplGetOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplGetOptions::RomaImplGetOptions() : m_signal(0)
{
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplGetOptions::RomaImplGetOptions(const RomaImplGetOptions& opt)
{
    *this = opt;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplGetOptions& 
RomaImplGetOptions::operator = (const RomaImplGetOptions& opt)
{
    if (this != &opt)
    {
        RomaImplCommonGetOptions::operator = (opt);
        m_signal = opt.m_signal;
        RomaImplProxy::getImpl(opt.m_security).populate(m_security);
    }

    return *this;
}

//--------------------------------------------------------------------
// Set the signal.
//--------------------------------------------------------------------
void RomaImplGetOptions::setSignal(KXCULONG signal)
{
    TRACE_ENTRY(RomaImplGetOptions::setSignal, << TRACE(this) 
                                               << TRACE(signal));

    useSignal(true);
    m_signal = signal;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaImplGetOptions::setSecurityDef(const RomaSecurityDef& def)
{
    TRACE_ENTRY(RomaImplGetOptions::setSecurityDef, << TRACE(this) 
                                                    << TRACE(def));

    RomaImplProxy::getImpl(def).populate(m_security);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Fill the C API structures with options data.
//--------------------------------------------------------------------
void RomaImplGetOptions::populate(KXCGETOPT& go, 
                                  KXCSYNC& sync, 
                                  HKXCTHEME& theme) const
{
    RomaImplCommonGetOptions::populate(go, sync, theme);
    go.Signal = m_signal;
    go.pSecurity = (PKXCSECDEF)RomaImplProxy::getImpl(m_security);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplGetOptions::trace(RomaTraceStream& s) const
{
    RomaImplCommonGetOptions::trace(s);
    s << TRACE(m_signal) << TRACE(m_security);
}


//====================================================================
// The RomaImplReportOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplReportOptions::RomaImplReportOptions()
    : m_default(Roma::ACTION_DEFAULT), m_disabled(0)
{
    *m_errorProfile = 0;
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaImplReportOptions::setErrorProfile(const char* name)
{
    copyZString(m_errorProfile, name, sizeof(m_errorProfile));
}

//--------------------------------------------------------------------
// Set report options.
//--------------------------------------------------------------------

void RomaImplReportOptions::setNoEms(bool b)
{
    if (b) m_disabled |= KXCEPOPT_NOEMS;
    else   m_disabled &= ~KXCEPOPT_NOEMS;
}

void RomaImplReportOptions::setNoNotification(bool b)
{
    if (b) m_disabled |= KXCEPOPT_NONOTIFY;
    else   m_disabled &= ~KXCEPOPT_NONOTIFY;
}

void RomaImplReportOptions::setNoTransformation(bool b)
{
    if (b) m_disabled |= KXCEPOPT_NOXFORM;
    else   m_disabled &= ~KXCEPOPT_NOXFORM;
}

void RomaImplReportOptions::setNoAlert(bool b)
{
    if (b) m_disabled |= KXCEPOPT_NOALERT;
    else   m_disabled &= ~KXCEPOPT_NOALERT;
}

//--------------------------------------------------------------------
// Set the message theme.
//--------------------------------------------------------------------
void RomaImplReportOptions::setTheme(HKXCTHEME theme)
{
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Fill the C API structures with options data.
//--------------------------------------------------------------------
void RomaImplReportOptions::populate(KXCEPOPT& ro, 
                                     KXCSYNC& sync, 
                                     HKXCTHEME& theme) const
{
    ro.ErrorOptions = m_disabled;
    ro.DefaultAction = m_default;

    // If there are any default actions, set the option to use them.
    if (m_default != Roma::ACTION_DEFAULT) 
    {
        ro.ErrorOptions |= KXCEPOPT_USEDEFAULT;
    }

    // If we asked for notification, make sure it is turned on.
    if ((m_default & Roma::ACTION_NOTIFY) != 0) 
    {
        ro.ErrorOptions &= ~KXCEPOPT_NONOTIFY;
    }

    if (*m_errorProfile) 
    {
        memcpy(ro.ErrorProfile, m_errorProfile, KXCOBJECT_NAME_LENGTH);
        ro.ErrorOptions |= KXCEPOPT_USEEPNAME;
    }

    RomaImplProxy::getImpl(m_syncpoint).populate(sync);
    theme = *m_theme;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplReportOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_HEX(m_default) << TRACE_HEX(m_disabled)
      << TRACE((char*)m_errorProfile) << TRACE(m_theme)
      << TRACE(m_syncpoint);
}

