//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoiopt.hpp,v 1.18 2004/01/22 17:53:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoiopt.hpp,v $
// $Revision: 1.18 $
//--------------------------------------------------------------------
// The internal implementation of the various Roma options objects.
//--------------------------------------------------------------------

#ifndef __KXCOIOPT_HPP__
#define __KXCOIOPT_HPP__

#include "kxco.hpp"
#include "kxcotrb.hpp"
#include "kxcosyn.hpp"
#include "kxcothm.hpp"
#include "kxcoils.hpp"
#include "kxcomid.hpp"
#include "kxcosec.hpp"
#include "kxcoigdt.hpp"


//--------------------------------------------------------------------
// The RomaImplOptions class.
// This is the base class for all options.
//--------------------------------------------------------------------
class KXCODLL RomaImplOptions : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplOptions(KXCULONG = 0);

    //----------------------------------------------------------------
    // Set an API profile name.
    //----------------------------------------------------------------
    void setApiProfile(const char*);

    //----------------------------------------------------------------
    // Set an error profile name.
    //----------------------------------------------------------------
    void setErrorProfile(const char*);

    //----------------------------------------------------------------
    // Get/set the message theme.
    //----------------------------------------------------------------
    const RomaTheme& getTheme() const { return m_theme; }
    void setTheme(const RomaTheme& theme) { m_theme = theme; }
    void setTheme(HKXCTHEME);

    //----------------------------------------------------------------
    // Set the syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& sp) { 
        m_syncpoint = sp; }

    //----------------------------------------------------------------
    // Set ILS data.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData&);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // API profile name.
    //----------------------------------------------------------------
    KXCOBJNAMESZ m_apiProfile;

    //----------------------------------------------------------------
    // Error profile name.
    //----------------------------------------------------------------
    KXCOBJNAMESZ m_errorProfile;

    //----------------------------------------------------------------
    // Roma options.
    //----------------------------------------------------------------
    KXCULONG m_options;

    //----------------------------------------------------------------
    // Status.
    //----------------------------------------------------------------
    KXCULONG m_status;

    //----------------------------------------------------------------
    // Message theme.
    //----------------------------------------------------------------
    RomaTheme m_theme;

    //----------------------------------------------------------------
    // Syncpoint definition.
    //----------------------------------------------------------------
    RomaSyncpointDef m_syncpoint;

    //----------------------------------------------------------------
    // ILS data handle.
    //----------------------------------------------------------------
    HKXCILS m_ilsOutgoing;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplCommonGetOptions class.
// The base class for 'get' options.
//--------------------------------------------------------------------
class KXCODLL RomaImplCommonGetOptions : public RomaImplOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplCommonGetOptions(KXCULONG = KXCGOPT_AUTO_ALLOC);

    //----------------------------------------------------------------
    // Set get options.
    //----------------------------------------------------------------
    void setNoEms(bool);
    void setNoNotification(bool);
    void setAutoAlloc(bool);
    void setAllowNative(bool);
    void setWaitIndefinite(bool);
    void setTermSupported(bool);
    void setSuppressErrorProfile(bool);
    void setCloseTheme(bool);
    void setRetainTheme(bool);
    void setUseTheme(bool);
    void setAnyTheme(bool);
    void useSignal(bool);

    //----------------------------------------------------------------
    // Set the operation.
    //----------------------------------------------------------------
    void setOperation(Roma::Operation);

    //----------------------------------------------------------------
    // Set the selector.
    //----------------------------------------------------------------
    void setSelector(Roma::GetSelector);

    //----------------------------------------------------------------
    // Set the theme control.
    //----------------------------------------------------------------
    void setThemeControl(Roma::ThemeControl);

    //----------------------------------------------------------------
    // Set the Message ID.
    //----------------------------------------------------------------
    void setMsgId(const RomaMsgId& mid) { m_msgId = mid; }

    //----------------------------------------------------------------
    // Set the Correlation ID.
    //----------------------------------------------------------------
    void setCorrelId(const RomaMsgId& cid) { m_correlId = cid; }

    //----------------------------------------------------------------
    // Set the message subject style.
    //----------------------------------------------------------------
    void setSubjectStyle(int style) { m_subjectStyle = style; }

    //----------------------------------------------------------------
    // Set the wait interval.
    //----------------------------------------------------------------
    void setWaitInterval(KXCULONG val) { m_waitTimeout = val; }

    //----------------------------------------------------------------
    // Fill the C API structures with options data.
    //----------------------------------------------------------------
    virtual void populate(KXCGETOPT&, KXCSYNC&, HKXCTHEME&) const;

    //----------------------------------------------------------------
    // Set options data from the C API structures.
    //----------------------------------------------------------------
    virtual void capture(const KXCGETOPT&, HKXCTHEME);
    
    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Wait interval.
    //----------------------------------------------------------------
    KXCULONG m_waitTimeout;

    //----------------------------------------------------------------
    // Subject style.
    //----------------------------------------------------------------
    KXCULONG m_subjectStyle;

    //----------------------------------------------------------------
    // Message ID.
    //----------------------------------------------------------------
    RomaMsgId m_msgId;

    //----------------------------------------------------------------
    // Correlation ID.
    //----------------------------------------------------------------
    RomaMsgId m_correlId;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplCommonPutOptions class.
// The base class for 'put' options.
//--------------------------------------------------------------------
class KXCODLL RomaImplCommonPutOptions : public RomaImplOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplCommonPutOptions();

    //----------------------------------------------------------------
    // Set put options.
    //----------------------------------------------------------------
    void setNewTheme(bool);
    void setUseTheme(bool);
    void setRetainTheme(bool);
    void setCloseTheme(bool);
    void setNoEms(bool);
    void setNoSyncAllowed(bool);
    void setNoNotification(bool);
    void setNotification(bool);
    void setNoTransformation(bool);
    void setTermSupported(bool);
    void setSuppressErrorProfile(bool);
    void setFreeBuffer(bool);
    void setDatagram(bool);
    void setCOD(bool);
    void setCOA(bool);
    void setIncrementRetryCount(bool);

    //----------------------------------------------------------------
    // Set the put selector.
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector);

    //----------------------------------------------------------------
    // Returns true if the theme is retained.
    //----------------------------------------------------------------
    bool isThemeRetained() const;

    //----------------------------------------------------------------
    // Fill the C API structures with options data.
    //----------------------------------------------------------------
    void populate(KXCPUTOPT&, KXCSYNC&, HKXCTHEME&) const;

    //----------------------------------------------------------------
    // Set options data from the C API structures.
    //----------------------------------------------------------------
    void capture(const KXCPUTOPT&, HKXCTHEME);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;
};


//--------------------------------------------------------------------
// The RomaImplExceptionOptions class.
// This is the internal implementation of the RomaExceptionOptions
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplExceptionOptions : public RomaImplCommonPutOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplExceptionOptions();

    //----------------------------------------------------------------
    // Set/get the business service name.
    //----------------------------------------------------------------
    void setServiceName(const char*);
    const char* getServiceName() const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Business service name.
    //----------------------------------------------------------------
    KXCOBJNAMESZ m_service;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplStoreOptions class.
// This is the internal implementation of the RomaStoreOptions
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplStoreOptions : public RomaImplCommonPutOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplStoreOptions() {}
};


//--------------------------------------------------------------------
// The RomaImplRetrieveOptions class.
// This is the internal implementation of the RomaRetrieveOptions
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplRetrieveOptions : public RomaImplCommonGetOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplRetrieveOptions();
};


//--------------------------------------------------------------------
// The RomaImplGetOptions class.
// This is the internal implementation of the RomaGetOptions
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplGetOptions : public RomaImplCommonGetOptions
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplGetOptions();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaImplGetOptions(const RomaImplGetOptions&);
    RomaImplGetOptions& operator = (const RomaImplGetOptions&);

    //----------------------------------------------------------------
    // Set the signal.
    //----------------------------------------------------------------
    void setSignal(KXCULONG);

    //----------------------------------------------------------------
    // Set the security definition.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef&);

    //----------------------------------------------------------------
    // Fill the C API structures with options data.
    //----------------------------------------------------------------
    virtual void populate(KXCGETOPT&, KXCSYNC&, HKXCTHEME&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Signal.
    //----------------------------------------------------------------
    KXCULONG m_signal;

    //----------------------------------------------------------------
    // Security definition.
    //----------------------------------------------------------------
    RomaSecurityDef m_security;

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplReportOptions class.
// This is the internal implementation of the RomaReportOptions
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplReportOptions : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplReportOptions();

    //----------------------------------------------------------------
    // Set report options.
    //----------------------------------------------------------------
    void setNoEms(bool);
    void setNoAlert(bool);
    void setNoNotification(bool);
    void setNoTransformation(bool);

    //----------------------------------------------------------------
    // Set/get default actions.
    //----------------------------------------------------------------
    void setDefaultActions(KXCULONG act) { m_default = act; }
    KXCULONG getDefaultActions() const { return m_default; }

    //----------------------------------------------------------------
    // Set the syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def) { 
        m_syncpoint = def; }

    //----------------------------------------------------------------
    // Set the message theme.
    //----------------------------------------------------------------
    void setTheme(const RomaTheme& theme) { m_theme = theme; }
    void setTheme(HKXCTHEME);

    //----------------------------------------------------------------
    // Set/get the error profile name.
    //----------------------------------------------------------------
    void setErrorProfile(const char*);
    const char* getErrorProfile() const { 
        return (const char*)m_errorProfile; }

    //----------------------------------------------------------------
    // Fill the C API structures with options data.
    //----------------------------------------------------------------
    void populate(KXCEPOPT&, KXCSYNC&, HKXCTHEME&) const;

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

protected:
    //----------------------------------------------------------------
    // Error profile name.
    //----------------------------------------------------------------
    KXCOBJNAMESZ m_errorProfile;

    //----------------------------------------------------------------
    // Default actions.
    //----------------------------------------------------------------
    KXCULONG m_default;

    //----------------------------------------------------------------
    // Disabled actions.
    //----------------------------------------------------------------
    KXCULONG m_disabled;

    //----------------------------------------------------------------
    // Message theme.
    //----------------------------------------------------------------
    RomaTheme m_theme;

    //----------------------------------------------------------------
    // Syncpoint definition.
    //----------------------------------------------------------------
    RomaSyncpointDef m_syncpoint;

    friend class KxcoStreamer;
};


#endif // __KXCOIOPT_HPP__

