//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2001-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosbs.cpp,v 1.10 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosbs.cpp,v $
// $Revision: 1.10 $
//--------------------------------------------------------------------
// The RomaSubscription public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcosbs.hpp>

TRACE_MODULE_VERSION($Revision: 1.10 $);


//--------------------------------------------------------------------
// Define RTTI information
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaSubscription);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaSubscription::RomaSubscription()
{
    TRACE_ENTRY(RomaSubscription::RomaSubscription, << TRACE(this));

    m_impl = new RomaImplSubscription;

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the business service name and the
// subject.
//--------------------------------------------------------------------
RomaSubscription::RomaSubscription(const char* serviceName,
                                   const RomaSubject& subject)
{
    TRACE_ENTRY(RomaSubscription::RomaSubscription, 
        << TRACE(this) << TRACE(serviceName) << TRACE(subject));

    m_impl = new RomaImplSubscription;
    m_impl->setServiceName(serviceName);
    m_impl->setSubject(subject);

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaSubscription::~RomaSubscription() 
{
    TRACE_ENTRY(RomaSubscription::~RomaSubscription, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaSubscription::RomaSubscription(const RomaSubscription& subs)
    : m_impl(subs.m_impl)
{
    TRACE_ENTRY(RomaSubscription::RomaSubscription, << TRACE(this)
                                                    << TRACE(subs));
    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaSubscription& 
RomaSubscription::operator = (const RomaSubscription& subs)
{
    TRACE_ENTRY(RomaSubscription::operator =, << TRACE(this)
                                              << TRACE(subs));

    if (this != &subs)
    {
        m_impl->releaseInstance();
        m_impl = subs.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Specify true to retrieve new publications only.
//--------------------------------------------------------------------
void RomaSubscription::setNewPubsOnly(bool flag)
{
    m_impl->setNewPubsOnly(flag);
}

//--------------------------------------------------------------------
// Specify true to retrieve publications by request only.
//--------------------------------------------------------------------
void RomaSubscription::setOnRequest(bool flag)
{
    m_impl->setOnRequest(flag);
}

//--------------------------------------------------------------------
// Set a business service name.
//--------------------------------------------------------------------
void RomaSubscription::setServiceName(const char* name)
{
    m_impl->setServiceName(name);
}

//--------------------------------------------------------------------
// Set theme handling options.
//--------------------------------------------------------------------
void RomaSubscription::setNewTheme(bool f)
{
    m_impl->setNewTheme(f);
}

void RomaSubscription::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

void RomaSubscription::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

void RomaSubscription::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Specify true to send the message as a datagram.
//--------------------------------------------------------------------
void RomaSubscription::setDatagram(bool f)
{
    m_impl->setDatagram(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of delivery" flag.
//--------------------------------------------------------------------
void RomaSubscription::setCOD(bool f)
{
    m_impl->setCOD(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of arrival" flag.
//--------------------------------------------------------------------
void RomaSubscription::setCOA(bool f)
{
    m_impl->setCOA(f);
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaSubscription::setSecurityDef(const RomaSecurityDef& secDef)
{
    m_impl->setSecurityDef(secDef);
}

//--------------------------------------------------------------------
// Get the theme.
//--------------------------------------------------------------------
const RomaTheme& RomaSubscription::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaSubscription::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Set the message priority.
//--------------------------------------------------------------------
void RomaSubscription::setPriority(int priority)
{
    m_impl->setPriority (priority);
}

//--------------------------------------------------------------------
// Set the message persistence.
//--------------------------------------------------------------------
void RomaSubscription::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Set the message feedback.
//--------------------------------------------------------------------
void RomaSubscription::setFeedback(int feedback)
{
    m_impl->setFeedback(feedback);
}

//--------------------------------------------------------------------
// Set the message expiry interval.
//--------------------------------------------------------------------
void RomaSubscription::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaSubscription::setMsgName(const char* name)
{
    m_impl->setMsgName(name);
}

//--------------------------------------------------------------------
// Set the message API profile name.
//--------------------------------------------------------------------
void RomaSubscription::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set the message error profile name.
//--------------------------------------------------------------------
void RomaSubscription::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Specify true to prohibit an EMS invocation on an error.
//--------------------------------------------------------------------
void RomaSubscription::setNoEms(bool f)
{
    m_impl->setNoEms(f);
}

//--------------------------------------------------------------------
// Set the termination indicator.
//--------------------------------------------------------------------
void RomaSubscription::setTermSupported(bool f)
{
    m_impl->setTermSupported(f);
}

//--------------------------------------------------------------------
// Specify true to suppress the error profile.
//--------------------------------------------------------------------
void RomaSubscription::setSuppressErrorProfile(bool f)
{
    m_impl->setSuppressErrorProfile(f);
}

//--------------------------------------------------------------------
// Set ILS data.
//--------------------------------------------------------------------
void RomaSubscription::setILSData(const RomaILSData& data)
{
    m_impl->setILSData(data);
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaSubscription::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Set a syncpoint definition.
//--------------------------------------------------------------------
void RomaSubscription::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Set a subject.
//--------------------------------------------------------------------
void RomaSubscription::setSubject(const RomaSubject& subject)
{
    m_impl->setSubject(subject);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaSubscription::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaSubscription, << TRACE (m_impl)) << "->";
    m_impl->trace(s);
}

