//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaConfig.java,v 1.2 2003/11/18 15:18:36 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaConfig.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaConfig.java
//
// Defines      : class RomaConfig
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the Roma environment configuration of the application.
*
* @see RomaExceptionInfo
*/
public final class RomaConfig
{
  /**
  * Returns the name of the application (client, server, or bridge) that
  * sent the message to the exception message handler.
  */
  public String getElementName()
  {
     return m_be;
  }

  /**
  * Returns the type of application that sent the message to
  * the exception message handler.
  *
  * @return one of <br><ul>
  * <li><code>Roma.APPTYPE_CLIENT</code> - the sender is a client</li>
  * <li><code>Roma.APPTYPE_SERVER</code> - the sender is a server</li>
  * <li><code>Roma.APPTYPE_BRIDGE</code> - the sender is a bridge</li>
  * </ul>
  */
  public int getElementType()
  {
     return m_etype;
  }

  /**
  * Returns the name of the applications input
  * (that is, response or request) queue.
  */
  public String getQueueName()
  {
     return m_queue;
  }

  /**
  * Returns the name of the business component reporting the error.
  * Meaningful only if the reporting application is a server
  * (that is, a business element).
  */
  public String getComponentName()
  {
     return m_bc;
  }

  /**
  * Returns the name of the business service of which this application
  * is a member. Meaningful only if the reporting application is a server
  * (that is, a business element).
  */
  public String getServiceName()
  {
     return m_bs;
  }

  /**
  * Returns the name of the node on which the error occurred.
  */
  public String getNodeName()
  {
     return m_node;
  }

  RomaConfig()
  {
  }

  RomaConfig(String be, int etype, String queue, String bc, String bs,
             String node)
  {
    if (be != null) m_be = be;
    if (queue != null) m_queue = queue;
    if (bc != null) m_bc = bc;
    if (bs != null) m_bs = bs;
    if (node != null) m_node = node;
    m_etype = etype;
  }

  private String m_node = "";
  private String m_bs = "";
  private String m_bc = "";
  private String m_queue = "";
  private String m_be = "";
  private int m_etype;
}

