//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaMessageInfo.java,v 1.2 2003/11/18 15:18:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaMessageInfo.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaMessageInfo.java
//
// Defines      : class RomaMessageInfo
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the message information. The information
* can be extracted with the
* {@link RomaBusinessElement#interpretMessage(RomaBeRequest,int)}
* or {@link RomaClient#interpretMessage(RomaClientResponse,int)}
* method.
*/
public final class RomaMessageInfo
{
  /**
  * Returns the associated workflow.
  */
  public RomaWorkflow getWorkflow()
  {
     return (m_wflow == null ? new RomaWorkflow() : m_wflow);
  }

  /**
  * Returns the theme.
  */
  public RomaTheme getTheme()
  {
     return (m_theme == null ? new RomaTheme() : m_theme);
  }

  /**
  * Returns the message data as Business Element request.
  */
  public RomaBeRequest getBeRequest()
  {
     if (m_msg == null) return new RomaBeRequest();
     return new RomaBeRequest(m_msg);
  }

  /**
  * Returns the message data as Client response.
  */
  public RomaClientResponse getClientResponse()
  {
     if (m_msg == null) return new RomaClientResponse();
     return new RomaClientResponse(m_msg);
  }

  /**
  * Returns the exception details.
  */
  public RomaExceptionInfo getException()
  {
     return (m_einfo == null ? new RomaExceptionInfo() : m_einfo);
  }

  /**
  * @return true if the theme information is available.
  */
  public boolean hasTheme()
  {
     return (m_theme != null);
  }

  /**
  * @return true if the message data is available.
  */
  public boolean hasMessage()
  {
     return (m_msg != null);
  }

  /**
  * @return true if there is the associated workflow.
  */
  public boolean hasWorkflow()
  {
     return (m_wflow != null);
  }

  /**
  * @return true if the exception information is available.
  */
  public boolean hasException()
  {
     return (m_einfo != null);
  }

  RomaMessageInfo(RomaWorkflow wf, long th,
                  RomaMessage msg, RomaExceptionInfo ei)
  {
     m_wflow = wf;
     m_einfo = ei;
     m_msg = msg;

     if (th != 0)
     {
        m_theme = new RomaTheme(th);
     }
  }

  private RomaWorkflow m_wflow;
  private RomaTheme m_theme;
  private RomaMessage m_msg;
  private RomaExceptionInfo m_einfo;
}

