//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaWorkflowService.java,v 1.2 2003/11/18 15:18:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaWorkflowService.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaWorkflowService.java
//
// Defines      : class RomaWorkflowService
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the business service information
* associated with the workflow.
*
* @see RomaWorkflow
*/
public class RomaWorkflowService implements Cloneable
{
  /**
  * Constructs the empty object.
  */
  public RomaWorkflowService()
  {
  }

  /**
  * Returns the name of the business service received the message.
  */
  public String getService()
  {
     return m_service;
  }

  /**
  * Sets the name of the business service sending the message.
  * This will be applied to the theme when calling
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * to change the workflow.
  *
  * @param sn Business Service name
  */
  public void setService(String sn)
  {
     m_service = (sn == null ? "" : sn);
  }

  /** 
  * Returns the name of the business component received the message.
  */
  public String getComponent()
  {
     return m_comp;
  }

  /** 
  * Sets the name of the business component sending the message.
  * This will be applied to the theme when calling
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * to change the workflow.
  *
  * @param cn Business Component name
  */
  public void setComponent(String cn)
  {
     m_comp = (cn == null ? "" : cn);
  }

  /**
  * Returns the name of the audit object associated with the business
  * service received the message.
  */
  public String getAudit()
  {
     return m_audit;
  }

  /**
  * Sets the audit object name to be used by the business
  * service sending the message.
  * This will be applied to the theme when calling
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * to change the workflow.
  *
  * @param an Audit object name
  */
  public void setAudit(String an)
  {
     m_audit = (an == null ? "" : an);
  }

  /**
  * Returns the name of the business service that has created or updated
  * the workflow.
  * This is the business service specified on a message put call
  * unless an application has changed it using
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * somewhere on the route,
  * in which case this is the business service that has changed the workflow.
  */
  public String getSourceService()
  {
     return m_bserv;
  }

  /**
  * Returns the name of the business flow that has updated the workflow.
  */
  public String getSourceFlow()
  {
     return m_bflow;
  }

  /**
  * Sets the name of the business flow.
  * This will be applied to the theme when calling
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * to change the workflow.
  *
  * @param flow Business Flow name
  */
  public void setSourceFlow(String flow)
  {
     m_bflow = (flow == null ? "" : flow);
  }

  /**
  * Returns the name of the operation that has updated the workflow.
  */
  public String getSourceOperation()
  {
     return m_oper;
  }

  /**
  * Sets the operation name.
  * This will be applied to the theme when calling
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * to change the workflow.
  *
  * @param op Operation name
  */
  public void setSourceOperation(String op)
  {
     m_oper = (op == null ? "" : op);
  }

  /**
  * Returns a copy of the object.
  */
  public Object clone()
  {
     RomaWorkflowService copy = null;

     try
     {
        copy = (RomaWorkflowService)super.clone();
        copy.m_service = new String(m_service);
        copy.m_comp = new String(m_comp);
        copy.m_audit = new String(m_audit);
        copy.m_bserv = new String(m_bserv);
        copy.m_bflow = new String(m_bflow);
        copy.m_oper = new String(m_oper);
     }
     catch (CloneNotSupportedException e)
     {
     }

     return copy;
  }


  RomaWorkflowService(String serv, String comp, String audit,
                      String bs, String bf, String oper)
  {
     if (serv != null) m_service = serv;
     if (comp != null) m_comp = comp;
     if (audit != null) m_audit = audit;
     if (bs != null) m_bserv = bs;
     if (bf != null) m_bflow = bf;
     if (oper != null) m_oper = oper;
  }

  private String m_service = "";
  private String m_comp = "";
  private String m_audit = "";
  private String m_bserv = "";
  private String m_bflow = "";
  private String m_oper = "";
}

