//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcConnection.java
//
// Defines      : class KxcConnection
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Provides Roma connection semantics.
* Connection provides client/server initilization, query native
* platform params (e.g. encoding) and other semantics (such as writeLog).
* Currently, only native Roma connections are supported.
* Provides functionality for initializing native connections (load library).
* Later, remote connection capability may be introduced by this class
* specialization.
*/
abstract public class KxcConnection
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2002";

  abstract public int platformEncoding();

  public KxcOper createNewOper( int nbrObjects )
  {
    KxcOper op  = newOper();
    // re-size to contain nbrObjects objects for op-code
    op.m_ip     = new int [nbrObjects];
    op.m_op     = new Object [nbrObjects];
    return op;
  }

  // creates new client and configures initialization operation
  // which should be called on the object upon return:
  //
  // RomaClient clnt = RomaConnection.nativeConnection().newClient(...);
  // clnt.oper().invoke();
  public KxcClient newClient( String  name,
                              int     opt,
                              byte    sec[],
                              String  modq,
                              long   ilsDataHandle,
                              String id,
                              String pwd)
  {
    return new KxcClient( newOper(), name, opt, sec, modq, ilsDataHandle,
           id, pwd );
  }

  // creates new server and configures initialization operation
  // which should be called on the object upon return:
  //
  // RomaServer svr = RomaConnection.nativeConnection().newServer(...);
  // svr.oper().invoke();
  public KxcServer newServer( String  name,
                              int     opt,
                              byte    sec[],
                              String  modq,
                              long    ilsDataHandle,
                              String id,
                              String pwd )
  {
    return new KxcServer( newOper(), name, opt, sec, modq, ilsDataHandle,
             id, pwd );
  }

  // Log writer may be used independetely from
  // context (client or server)
  public class LogWriter extends KxcLogData
  {
    private final String copyrightNotice =
      "Copyright Candle Corporation 1999-2002";
    
    LogWriter( KxcOper oper )
    {
      super( oper );
      m_oper=oper;
    }

    // configures write log operation
    public KxcOper writeLog()
    {
      m_ip[OP_CODE] = NIOP_WRITELOG;
      return m_oper;
    }

    // configures write log operation
    public KxcOper writeSysLog()
    {
      m_ip[OP_CODE] = NIOP_WRITESYSLOG;
      return m_oper;
    }
    private KxcOper m_oper;
  };

  
  // creates and returns new log writer
  public LogWriter logWriter()
  {
    return new LogWriter( newOper() );
  }
  
  /**
  * Returns native connection object.
  * The object is allocated on the first call.
  */
  public static KxcConnection nativeConnection()
  {
    if ( ms_nativeConn == null )
    {
      System.loadLibrary("kxcjni");
      ms_nativeConn = new KxcNativeConnection();            
    }

    return ms_nativeConn;
  }
  
  static KxcConnection ms_nativeConn = null;
  
  abstract protected KxcOper newOper();
}

//--------------------------------------------------------------------
// End of file KxcConnection.java
//--------------------------------------------------------------------
