:begin
@echo off

REM 
REM $Source: /romacvs/root/roma/src/kxc/kxcntgathr.bat,v $
REM $Revision: 1.2 $
REM Unpublished copyright 1999-2003, Candle Corporation, a California 
REM corporation.  All rights reserved. International copyright secured.
REM
REM This material is proprietary to Candle Corporation and is not to be
REM reproduced, used, or disclosed except in accordance with program
REM licenses or upon written authorization of Candle Corporation.
REM
REM kxcgathr.bat - Collect system info for CASP problems
REM
REM	Information collected:
REM       Candle registry info
REM       CASP installation logs
REM       CASP error logs
REM       LDAP dump
REM       O/S information
REM       Environment info
REM       CASP installation directory contents
REM       MQSeries version
REM       MQSeries configuration
REM       MQSeries error logs
REM       Dr Watson logs
REM       Network info
REM
REM	The resulting files are placed in the CASP logs directory and
REM	compressed using (kru)gzip.  Compressed files have a .gz extension.
REM
REM


REM
REM Make sure directories exist
REM
if exist "%CANDLE_CASP_HOME%" goto setlog
echo CANDLE_CASP_HOME must be set before running kxcgathr.
goto end
remexit

if exist "%CANDLE_COMMON_HOME%" goto setlog
echo CANDLE_COMMON_HOME must be set before running kxcgathr.
goto end
remexit

:setlog
set LOG_DIR="%CANDLE_CASP_HOME%"\logs
set COMMON_LOG_DIR="%CANDLE_COMMON_HOME%"\logs
set OUT_DIR="%CANDLE_CASP_HOME%"\logs
set BIN_DIR="%CANDLE_CASP_HOME%"\toolkit\bin
if not exist %OUT_DIR% goto makegatherdir
goto registry

:makegatherdir
mkdir %OUT_DIR%


REM
REM Get registry info for Candle products
REM
:registry
echo Gathering registry...
regedit /e candle.reg HKEY_LOCAL_MACHINE\SOFTWARE\Candle
if not exist candle.reg goto install
move candle.reg %OUT_DIR%


REM
REM Get CASP installation logs
REM
:install
echo Gathering eBP installation logs...
if not exist "%CANDLE_CASP_HOME%"\Roma_installation.log goto grabFFDC
xcopy /q "%CANDLE_CASP_HOME%"\Roma_installation.log %OUT_DIR%
echo off


REM
REM Get CASP FFDC logs
REM
:grabFFDC
echo Gathering CASP error logs...
if not exist %BIN_DIR%\*.FDC goto dumpLDAP
xcopy /q %BIN_DIR%\*.FDC %OUT_DIR%


REM
REM Get contents of LDAP directory
REM
:dumpLDAP
echo Gathering LDAP info...
REM ldapsearch -v -b "" -s sub "(objectclass=*)" > %OUT_DIR%\ldapdump.out
krugldap -v > %OUT_DIR%\ldapdump.out


REM
REM Get system info
REM
:winmsd 
echo Gathering system info...
winmsd /a 
if not exist %COMPUTERNAME%.TXT goto dirROMA
move %COMPUTERNAME%.TXT %OUT_DIR%


REM
REM Get contents of installation directory
REM
:dirROMA
echo Gathering CASP installation directory...
if not exist "%CANDLE_CASP_HOME%" goto mqmtransport
dir /s "%CANDLE_CASP_HOME%" > %OUT_DIR%\candledir.out


REM
REM Get MQSeries info
REM
:mqmtransport
echo Gathering MQSeries info...
if not exist \mqm goto msmqtransport
:mqmtransportreadme
xcopy /q \mqm\readme.txt %OUT_DIR%
:mqmtransportconfig
xcopy /q \mqm\mqs.ini %OUT_DIR%
:mqmtransporterrors
xcopy /q \mqm\errors\*.* %OUT_DIR%

:msmqtransport


REM
REM Get Dr Watson logs
REM
:drwatson
echo Gathering Dr Watson logs...
if exist c:\winnt\user.dmp xcopy /q c:\winnt\user.dmp %OUT_DIR%
if exist c:\winnt\DRWTSN32.LOG xcopy /q c:\winnt\DRWTSN32.LOG %OUT_DIR%


REM
REM Get network info
REM
:netinfo
echo Gathering network info...
netstat -aenrs > %OUT_DIR%\netstat.out
ipconfig /all > %OUT_DIR%\ipconfig.out


REM
REM Run component-specific scripts
REM
:specific
echo Running component-specific scripts...
pushd "%CANDLE_CASP_HOME%"\toolkit\bin
for %%x in (???gathr.bat ) do if not %%x == kxcgathr.bat call %%x
popd

pushd "%CANDLE_COMMON_HOME%"\toolkit\bin
for %%x in (???gathr.bat ) do if not %%x == kxcgathr.bat call %%x
popd


REM
REM Compress files
REM
:compress
echo Compressing files...
pushd %OUT_DIR%
if exist gather.old.gz erase gather.old.gz
if exist gather.gz move gather.gz gather.old.gz
echo. > gather
echo. > gathinfo
echo /**********************************************************************/ >> gather
echo. >> gather
echo                          CASP WinNT Gather File >> gather           
echo. >> gather
echo /**********************************************************************/ >> gather
echo. >> gather
echo Contents: >> gather           
echo. >> gather
echo   Files included >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		Files included >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
dir /TW /OD >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
if not exist candledir.out goto nocandledir
echo   CASP directory tree contents >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		CASP directory tree contents >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type candledir.out >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nocandledir
if not exist Roma_installation.log goto noromainstall
echo   CASP installation log >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		CASP installation log >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type Roma_installation.log >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:noromainstall
if not exist candle.reg goto noreg
echo   Registry entries (HKEY_LOCAL_MACHINE\SOFTWARE\Candle) >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		Registry entries >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type candle.reg >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:noreg
if not exist ldapdump.out goto noldap
echo   LDAP Directory >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		LDAP Directory >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type ldapdump.out >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:noldap
REM
REM we don't know where the FFDC files are. krugffdc figures that out, and formats
REM them before stuffing them into the gather file...
REM
echo   FFDC files (if any) >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		FFDC files >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
krugffdc gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
if not exist readme* goto nomqmread
echo   MQM readme.txt >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		MQM Readme file >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
if exist readme type readme >> gathinfo
if exist readme.txt type readme.txt >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nomqmread
if not exist mqs.ini goto nomqmini
echo   MQM mqs.ini file >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		MQM mqs.ini file >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type mqs.ini >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nomqmini
if not exist \mqm\errors\*.* goto nomqmerrs
echo   MQM error files >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		MQM error files >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
for %%x in ( \mqm\errors\*.log ) do type %%x >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nomqmerrs
if not exist %COMPUTERNAME%.txt goto nomsdstuff
echo   WinMSD report for %COMPUTERNAME% >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		WinMSD report for %COMPUTERNAME% >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type %COMPUTERNAME%.txt >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nomsdstuff
if not exist DRWTSN32.LOG goto nodoctor
echo   Dr. Watson log >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		Dr. Watson log >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type DRWTSN32.LOG >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nodoctor
if not exist ipconfig.out goto noipconfig
echo   ipconfig output >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		ipconfig output >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type ipconfig.out >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:noipconfig
if not exist netstat.out goto nonetstat
echo   netstat output >> gather
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
echo		netstat output >> gathinfo           
echo. >> gathinfo
echo /**********************************************************************/ >> gathinfo
echo. >> gathinfo
type netstat.out >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
echo. >> gathinfo
:nonetstat

REM
REM since the app specific gather files are built before the general gather, we have to stuff them 
REM away until the general gather and gather toc files are constructed; then we glue them onto the
REM end... 
REM

if not exist specific goto nonspecific
if not exist specinfo goto nonspecific

type specific >> gather

echo. >> gather
echo. >> gather
echo. >> gather

type specinfo >> gathinfo

erase specific
erase specinfo

:nonspecific

type gathinfo >> gather

erase gathinfo

:cleanup

erase *.dmp 2> nul
erase *.out 2> nul
erase *.reg 2> nul
erase *.fdc 2> nul
erase *.txt 2> nul
erase *.log 2> nul
erase *.ini 2> nul
erase readme*.* 2> nul

echo.
echo.
echo Gather complete.  Please send the files in
echo %CANDLE_CASP_HOME%\logs
dir /B %CANDLE_CASP_HOME%\logs
echo %CANDLE_COMMON_HOME%\logs
dir /B %CANDLE_COMMON_HOME%\logs
echo to Candle Corporation Technical Support for analysis.
echo.
echo.
popd

:end

