//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcomid.hpp,v 1.12 2003/12/11 18:30:07 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcomid.hpp,v $
// $Revision: 1.12 $
//--------------------------------------------------------------------
// The RomaMsgId public interface.
//--------------------------------------------------------------------

#ifndef __KXCOMID_HPP
#define __KXCOMID_HPP

#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplMsgId;


//---------------------------------------------------------------------
// The RomaMsgId class.
//
// The RomaMsgId class represents a binary identifier of a fixed size.
// A Roma Message has two such identifiers:
//      * message identifier
//      * correlation identifier
//---------------------------------------------------------------------
class KXCODLL RomaMsgId : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructor.
    // The ID initialized to null.
    //----------------------------------------------------------------
    RomaMsgId();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaMsgId();

    //----------------------------------------------------------------
    // Constructs the ID from binary data.
    //----------------------------------------------------------------
    RomaMsgId(const void* data, int length);

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaMsgId(const RomaMsgId& msgId);
    RomaMsgId& operator = (const RomaMsgId& msgId);

    //----------------------------------------------------------------
    // Assigns the ID with a byte array. If "length" is smaller than
    // Roma::ID_LENGTH, fills remaining bytes with 0.
    //
    // Parameters:
    //   (in) data - byte array
    //   (in) length - number of bytes
    //----------------------------------------------------------------
    void set(const void* data, int length = Roma::ID_LENGTH);

    //----------------------------------------------------------------
    // Returns true if two msg ids match, false otherwise.
    //----------------------------------------------------------------
    bool operator == (const RomaMsgId& msgId) const;

    //----------------------------------------------------------------
    // Returns true if two msg ids don't match, false otherwise.
    //----------------------------------------------------------------
    bool operator != (const RomaMsgId& msgId) const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;
    operator const void*() const;
    const char* operator *() const; // deprecated.

protected:
    RomaMsgId(RomaImplMsgId*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplMsgId* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOMID_HPP

