//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcorvo.hpp,v 1.11 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcorvo.hpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaRetrieveOptions public interface.
//--------------------------------------------------------------------

#ifndef __KXCORVO_HPP
#define __KXCORVO_HPP

#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplRetrieveOptions;


//--------------------------------------------------------------------
// The RomaRetrieveOptions class.
//
// Represents the options used when retrieving a message from a
// storage queue.
//--------------------------------------------------------------------
class KXCODLL RomaRetrieveOptions : public RomaTraceable
{
public:
    //-----------------------------------------------------------------
    // Constructor and destructor.
    //-----------------------------------------------------------------
    RomaRetrieveOptions();
    virtual ~RomaRetrieveOptions();

    //-----------------------------------------------------------------
    // Copy and assignment.
    //-----------------------------------------------------------------
    RomaRetrieveOptions(const RomaRetrieveOptions&);
    RomaRetrieveOptions& operator = (const RomaRetrieveOptions&);

    //----------------------------------------------------------------
    // Set an API profile name.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Set an error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // Define the get operation, as one of:
    //    GET_FIRST  - get first message  (destructive read)
    //    PEEK_FIRST - peek first message (non-destructive)
    //    PEEK_NEXT  - peek next message
    //    PEEK_SAME  - peek message under current read position
    //    GET_PEEKED - get message under current read position
    //
    // All operations work with messages which match the selector.
    // See setSelector() for selector option.
    //
    // Can be overidden by API profile.
    //----------------------------------------------------------------
    void setOperation(Roma::Operation operation);

    //----------------------------------------------------------------
    // Define the selection criteria for the get operation, as one of:
    //   BY_NONE     - get any message
    //   BY_THEME    - match on theme, as specified by setTheme
    //   BY_MSGID    - match on MessageId, as specified by setMsgId()
    //   BY_CORRELID - match on CorrelationId
    //   BY_MSGID_AND_CORRELID - BY_MSGID + BY_CORRELID
    //   BY_MSGID_AND_THEME    - BY_MSGID + BY_THEME
    //
    // Can be overidden by API profile.
    //----------------------------------------------------------------
    void setSelector(Roma::GetSelector selector);

    //----------------------------------------------------------------
    // Set actual value to use in conjunction with specified selector.
    //----------------------------------------------------------------
    void setMsgId(const RomaMsgId& msgId);
    void setCorrelId(const RomaMsgId& correlId);
    void setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Specify how theme is handled after the retrieve operation.
    //----------------------------------------------------------------
    void setCloseTheme  (bool f = true);
    void setRetainTheme (bool f = true);
    void setUseTheme    (bool f = true);
    void setAnyTheme    (bool f = true);

    //----------------------------------------------------------------
    // Sets number of milliseconds to wait for the message, or specify
    // that the operation should wait indefinitely.
    //
    // Can be overidden by API profile.
    //----------------------------------------------------------------
    void setWaitInterval(int milliseconds);
    void setWaitIndefinite(bool f = true);

    //----------------------------------------------------------------
    // If true, Roma will not allow the option to send the message
    // to the Exception Message Queue if the Get operation fails.
    //
    // Can be overidden by API profile.
    //----------------------------------------------------------------
    void setNoEms(bool f = true);

    //----------------------------------------------------------------
    // If true, Roma will not allow the option to send a
    // notification message back to the client if the Get
    // operation fails.
    //
    // Can be overidden by API profile.
    //----------------------------------------------------------------
    void setNoNotification(bool f = true);

    //----------------------------------------------------------------
    // Set a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplRetrieveOptions* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCORVO_HPP

