//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoso.hpp,v 1.11 2003/02/21 22:48:08 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoso.hpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaStoreOptions public interface.
//--------------------------------------------------------------------

#ifndef __KXCOSO_HPP
#define __KXCOSO_HPP

#include <kxco.hpp>
#include <kxcothm.hpp>
#include <kxcosyn.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplStoreOptions;


//--------------------------------------------------------------------
// The RomaStoreOptions class.
//
// The options used when storing a message to a storage queue.
//--------------------------------------------------------------------
class KXCODLL RomaStoreOptions : public RomaTraceable
{
public:
    //-----------------------------------------------------------------
    // Constructor and destructor.
    //-----------------------------------------------------------------
    RomaStoreOptions();
    virtual ~RomaStoreOptions();

    //-----------------------------------------------------------------
    // Copy and assignment.
    //-----------------------------------------------------------------
    RomaStoreOptions(const RomaStoreOptions&);
    RomaStoreOptions& operator = (const RomaStoreOptions&);

    //----------------------------------------------------------------
    // Sets the store selector.
    //
    // The selector specifies which identifiers contained in a
    // Roma message (msg id, correl id) should be when the message
    // is sent.
    //
    // The selector is specified by one of the following constants:
    //   USE_NONE     - doesn't set any specific selector
    //   USE_MSGID    - use Message Id as the selector
    //   USE_CORRELID - use Correlation Id as the selector
    //   USE_MSGID_AND_CORRELID - use both
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector selector);

    //----------------------------------------------------------------
    // Allows/disallows putting messages under transaction to a
    // non-transactional queue.
    //----------------------------------------------------------------
    void setNoSyncAllowed(bool f = true);

    //----------------------------------------------------------------
    // Set an API profile name.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Set theme options.
    //
    // If a new theme is requested, store() uses a new theme for each
    // message stored. Otherwise the specified theme will be used.
    //
    // If close theme is requested, the theme is closed after the
    // message is sent. If retain theme is requested, the theme is
    // retained for future use.
    //
    // These selections may be overidden if an API profile is defined.
    //----------------------------------------------------------------
    void setNewTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setCloseTheme(bool f = true);
    void setRetainTheme(bool f = true);

    //----------------------------------------------------------------
    // Set a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplStoreOptions* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOSO_HPP

