//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosyn.hpp,v 1.8 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosyn.hpp,v $
// $Revision: 1.8 $
//--------------------------------------------------------------------
// The RomaSyncpointDef public interface.
//--------------------------------------------------------------------

#ifndef __KXCOSYN_HPP
#define __KXCOSYN_HPP

#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplSyncpoint;


//--------------------------------------------------------------------
// The RomaSyncpointDef class.
//
// Specifies a syncpoint type and optionally a context, which are used
// during put() and get() operations.
//--------------------------------------------------------------------
class KXCODLL RomaSyncpointDef : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructs a syncpoint definition with specified type.
    //----------------------------------------------------------------
    RomaSyncpointDef(Roma::SyncType type = Roma::SYNC_PROFILE);

    //----------------------------------------------------------------
    // Constructs the global syncpoint definition.
    //----------------------------------------------------------------
    RomaSyncpointDef(PKXCVOID context);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaSyncpointDef();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaSyncpointDef(const RomaSyncpointDef&);
    RomaSyncpointDef& operator = (const RomaSyncpointDef&);

    //----------------------------------------------------------------
    // Set the syncpoint type.
    //
    // Possible values:
    //   SYNC_NONE      - no syncpoint to be used.
    //   SYNC_LOCAL     - specifies that syncpointing is under
    //                    Roma's control.
    //   SYNC_GLOBAL    - when sending and receiving messages, this is
    //                    similar to LOCAL syncpointing, except that
    //                    Roma assumes that the syncpoint is being
    //                    controlled outside of Roma (e.g. by CICS).
    //   SYNC_PROFILE   - Roma uses the value specified by the API
    //                    profile. If the API profile does not contain
    //                    a value for this field, Roma uses
    //                    the default (SYNC_NONE).
    //----------------------------------------------------------------
    void setType(Roma::SyncType type);

    //----------------------------------------------------------------
    // Set a context of the gloabl syncpoint.
    //
    // When applications use GLOBAL syncpointing, Roma passes the
    // "context" attribute to the Transport layer.
    //----------------------------------------------------------------
    void setContext(PKXCVOID context);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplSyncpoint* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOSYN_HPP

