//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcots.hpp,v 1.9 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcots.hpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaTimestamp public interface.
//--------------------------------------------------------------------

#ifndef __KXCOTS_HPP
#define __KXCOTS_HPP

#include <kxco.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplTimestamp;


//--------------------------------------------------------------------
// The RomaTimestamp class.
//
// The number of seconds that have elapsed since midnight on
// January 1st, 1970 C.U.T.
//
// The number of milliseconds that have expired since the last
// full second (that is, this is a number between 0 and 999).
//--------------------------------------------------------------------
class KXCODLL RomaTimestamp
{
public:
    //----------------------------------------------------------------
    // Constructor and destructor.
    //----------------------------------------------------------------
    RomaTimestamp(KXCULONG seconds = 0, KXCULONG milliseconds = 0);
    virtual ~RomaTimestamp();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaTimestamp(const RomaTimestamp& timestamp);
    RomaTimestamp& operator = (const RomaTimestamp& timestamp);

    //----------------------------------------------------------------
    // Comparison operators.
    //----------------------------------------------------------------
    bool operator == (const RomaTimestamp& timestamp) const;
    bool operator != (const RomaTimestamp& timestamp) const;
    bool operator < (const RomaTimestamp& timestamp) const;
    bool operator <= (const RomaTimestamp& timestamp) const;
    bool operator > (const RomaTimestamp& timestamp) const;
    bool operator >= (const RomaTimestamp& timestamp) const;

    //----------------------------------------------------------------
    // Attribute accessors.
    //----------------------------------------------------------------
    KXCULONG seconds() const;
    void setSeconds(KXCULONG seconds);
    KXCULONG milliseconds() const;
    void setMilliseconds(KXCULONG milliseconds);

    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    // Exchange with KXCTIMESTAMP.
    //----------------------------------------------------------------
    void set(const KXCTIMESTAMP& timestamp);
    void get(KXCTIMESTAMP& timestamp) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplTimestamp* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOTS_HPP

