/*********************************************************************
 * $Id: kxmchf.h,v 1.66 2003/12/09 13:30:10 e_ab Exp $
 * $Source: /romacvs/root/roma/src/kxm/kxmchf/kxmchf.h,v $
 *********************************************************************
 * Description: KXM C Helper Functions external definitions
 *********************************************************************
 *                                                                    
 * Copyright 1999-2003, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************/

/********************************************************************/
/* Protect against multiple includes                                */
/********************************************************************/
#ifndef KXMCHF_INCLUDED
#define KXMCHF_INCLUDED

#include <kxmenv.h>
#include <kxmerror.h>

/********************************************************************/
/* Enable C++ support                                               */
/********************************************************************/
#if defined(__cplusplus)
  extern "C" {
#endif

/********************************************************************/
/* This macro creates signature number from two letters             */
/********************************************************************/
#ifndef KXMSIG
#define KXMSIG(a,b)             (('X'<<24)|                  \
                                (((unsigned char)'M')<<16)|  \
                                (((unsigned char)a)<<8)|     \
                                ((unsigned char)b))
#endif

/********************************************************************/
/* Types                                                            */
/********************************************************************/

#define KXMOBJECT_NAME_LENGTH 256

typedef char KXMCHAR;
typedef KXMCHAR KXMFORMATNAMESZ[KXMOBJECT_NAME_LENGTH+1];
typedef KXMCHAR KXMFIELDNAMESZ[KXMOBJECT_NAME_LENGTH+1];
typedef unsigned short KXMUCCHAR;

/********************************************************************/
/* Handles                                                          */
/********************************************************************/

typedef struct KxmCFieldContainer* HKXMMSG;
typedef struct KxmCFieldContainer* HKXMFIELD;

/********************************************************************/
/* Initialization options                                           */
/********************************************************************/

#define KXMINITOPT_NONE         0
#define KXMINITOPT_STANDALONE   1

#define KXMSIG_INITOPT          KXMSIG('I','N')
#define KXMVER_INITOPT_1        1
#define KXMVER_INITOPT_CURRENT  1

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
} KXMINITOPT;

#define KXMINITOPT_DEFAULT      KXMSIG_INITOPT,\
                                KXMVER_INITOPT_CURRENT,\
                                KXMINITOPT_NONE

/********************************************************************/
/* Encoding                                                         */
/********************************************************************/
#define KXMSIG_ENC              KXMSIG('E','C')
#define KXMVER_ENC_1            1
#define KXMVER_ENC_2            2
#define KXMVER_ENC_CURRENT      KXMVER_ENC_2

#define KXMENC_BYTEORDER_DEFAULT 0
#define KXMENC_BYTEORDER_NATIVE  1
#define KXMENC_BYTEORDER_LE      2
#define KXMENC_BYTEORDER_BE      3

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    char       CodePage[64];
    KXMULONG   ByteOrder;
    KXMULONG   Reserved2;
    KXMULONG   Reserved3;
} KXMENC;

#define KXMENC_DEFAULT          KXMSIG_ENC,\
                                KXMVER_ENC_CURRENT,\
                                "",\
                                KXMENC_BYTEORDER_DEFAULT,\
                                0, 0

/********************************************************************/
/* Message options                                                  */
/********************************************************************/

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
} KXMMSGOPT_1;

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
    KXMENC   Encoding;
} KXMMSGOPT_2;

typedef struct
{
    KXMULONG        Signature;
    KXMULONG        Version;
    KXMULONG        Options;
    KXMENC          Encoding;
    const KXMCHAR*  NsUri;
    KXMLONG         NsUriLength;
} KXMMSGOPT;

#define KXMMSGOPT_NONE                  0
#define KXMMSGOPT_STANDALONE            1
#define KXMMSGOPT_MDR                   2
#define KXMMSGOPT_CONVERT_BY_MAP        4
#define KXMMSGOPT_ANY                   8
#define KXMMSGOPT_FORMAT_NAME           16
#define KXMMSGOPT_ARRAY                 32
#define KXMMSGOPT_HANDLE_NAMESPACES     64
#define KXMMSGOPT_OLD_TEXT_FORMAT       128
#define KXMMSGOPT_PRESERVE_XML_SPACES   256
#define KXMMSGOPT_DISABLE_NAMESPACES    512
#define KXMMSGOPT_HEX_XML_FORMAT       1024
#define KXMMSGOPT_SCALAR_AS_ARRAY      2048

#define KXMSIG_MSGOPT           KXMSIG('M','O')
#define KXMVER_MSGOPT_1         1
#define KXMVER_MSGOPT_2         2
#define KXMVER_MSGOPT_3         3
#define KXMVER_MSGOPT_CURRENT   3

#define KXMMSGOPT_DEFAULT_1     KXMSIG_MSGOPT,\
                                KXMVER_MSGOPT_1,\
                                KXMMSGOPT_NONE

#define KXMMSGOPT_DEFAULT_2     KXMSIG_MSGOPT,\
                                KXMVER_MSGOPT_2,\
                                KXMMSGOPT_NONE,\
                                { KXMENC_DEFAULT }

#define KXMMSGOPT_DEFAULT_3     KXMSIG_MSGOPT,\
                                KXMVER_MSGOPT_3,\
                                KXMMSGOPT_NONE,\
                                { KXMENC_DEFAULT },\
                                NULL, -1

#define KXMMSGOPT_DEFAULT       KXMMSGOPT_DEFAULT_3


/********************************************************************/
/* Convert options                                                  */
/********************************************************************/

#define KXMCNVOPT_NONE                   0
#define KXMCNVOPT_TO_XML                 1
#define KXMCNVOPT_TO_FORMAT              2
#define KXMCNVOPT_BY_MAP                 4
#define KXMCNVOPT_MAP_REQUIRED           8
#define KXMCNVOPT_HANDLE_NAMESPACES     64
#define KXMCNVOPT_OLD_TEXT_FORMAT      128
#define KXMCNVOPT_DISABLE_NAMESPACES   512
#define KXMCNVOPT_HEX_XML_FORMAT      1024
#define KXMCNVOPT_SCALAR_AS_ARRAY     2048

/********************************************************************/
/* Find field options                                               */
/********************************************************************/

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
} KXMFLDOPT_1;

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
    KXMULONG FieldFormat;
} KXMFLDOPT_2;

typedef struct
{
    KXMULONG Signature;
    KXMULONG Version;
    KXMULONG Options;
    KXMULONG FieldFormat;
    KXMULONG Property;
} KXMFLDOPT;

#define KXMFLDOPT_FIELDFORMAT_NONE            0
#define KXMFLDOPT_FIELDFORMAT_BINARY_BASE64   1
#define KXMFLDOPT_FIELDFORMAT_BINARY_HEX      2

#define KXMFLDOPT_NONE                  0
#define KXMFLDOPT_EXACT                 2
#define KXMFLDOPT_SEARCH                4
#define KXMFLDOPT_UNIQUE                8
#define KXMFLDOPT_NUMBER                16
#define KXMFLDOPT_MILLISECONDS          32
#define KXMFLDOPT_MICROSECONDS          64
#define KXMFLDOPT_GET_SCHEMA_FIELDS     128

#define KXMFLDOPT_PROPERTY_NONE         0
#define KXMFLDOPT_PROPERTY_CHAR_DATA    1
#define KXMFLDOPT_PROPERTY_NIL          2

#define KXMSIG_FLDOPT           KXMSIG('F','O')
#define KXMVER_FLDOPT_1         1
#define KXMVER_FLDOPT_2         2
#define KXMVER_FLDOPT_3         3
#define KXMVER_FLDOPT_CURRENT   KXMVER_FLDOPT_3

#define KXMFLDOPT_DEFAULT_1     KXMSIG_FLDOPT,\
                                KXMVER_FLDOPT_1,\
                                KXMFLDOPT_NONE

#define KXMFLDOPT_DEFAULT_2     KXMSIG_FLDOPT,\
                                KXMVER_FLDOPT_2,\
                                KXMFLDOPT_NONE,\
                                KXMFLDOPT_FIELDFORMAT_NONE

#define KXMFLDOPT_DEFAULT       KXMSIG_FLDOPT,\
                                KXMVER_FLDOPT_CURRENT,\
                                KXMFLDOPT_NONE,\
                                KXMFLDOPT_FIELDFORMAT_NONE,\
                                KXMFLDOPT_PROPERTY_NONE

/********************************************************************/
/* Buffer                                                           */
/********************************************************************/

#define KXMBUFOWN_USER          0
#define KXMBUFOWN_KXM           1

#define KXMSIG_BUFFER           KXMSIG('B','F')
#define KXMVER_BUFFER_1         1
#define KXMVER_BUFFER_2         2
#define KXMVER_BUFFER_CURRENT   2

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    void*      Buffer;
    KXMULONG   BufferLen;
    KXMULONG   DataLen;
    KXMULONG   Owner;
} KXMBUFFER_1;

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    void*      Buffer;
    KXMULONG   BufferLen;
    KXMULONG   DataLen;
    KXMULONG   Owner;
    KXMENC     Encoding;
} KXMBUFFER;

#define KXMBUFFER_DEFAULT_1     KXMSIG_BUFFER,\
                                KXMVER_BUFFER_1,\
                                NULL, 0, 0,\
                                KXMBUFOWN_KXM

#define KXMBUFFER_DEFAULT       KXMSIG_BUFFER,\
                                KXMVER_BUFFER_CURRENT,\
                                NULL, 0, 0,\
                                KXMBUFOWN_KXM,\
                                { KXMENC_DEFAULT }

/********************************************************************/
/* Extract options                                                  */
/********************************************************************/

#define KXMEXTOPT_NONE          0
#define KXMEXTOPT_AUTOALLOC     1
#define KXMEXTOPT_NO_MARGINS    2

#define KXMSIG_EXTOPT           KXMSIG('E','O')
#define KXMVER_EXTOPT_1         1
#define KXMVER_EXTOPT_2         2
#define KXMVER_EXTOPT_CURRENT   2

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    KXMULONG   Options;
} KXMEXTOPT_1;

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    KXMULONG   Options;
    KXMENC     Encoding;
} KXMEXTOPT;

#define KXMEXTOPT_DEFAULT_1     KXMSIG_EXTOPT,\
                                KXMVER_EXTOPT_1,\
                                KXMEXTOPT_NONE

#define KXMEXTOPT_DEFAULT       KXMSIG_EXTOPT,\
                                KXMVER_EXTOPT_CURRENT,\
                                KXMEXTOPT_NONE,\
                                { KXMENC_DEFAULT }

/********************************************************************/
/* Error structure                                                  */
/********************************************************************/

#define KXMETYPE_API            0
#define KXMETYPE_TRANSIENT      1
#define KXMETYPE_ERROR          2
#define KXMETYPE_CRITICAL       3

#define KXMSIG_ERROR            KXMSIG('E','R')
#define KXMVER_ERROR_1          1
#define KXMVER_ERROR_2          2
#define KXMVER_ERROR_CURRENT    2

#define KXMERROR_MSG_LEN        400
#define KXMLOCATION_LEN         128

#define KXMVER_SUBERROR_1       1
#define KXMVER_SUBERROR_2       2
#define KXMVER_SUBERROR_CURRENT 2
#define KXMSIG_SUBERROR         KXMSIG('S','E')

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    KXMULONG   ReturnCode;
    KXMULONG   OpType;
    KXMULONG   Reason;
    KXMULONG   Position;
    KXMULONG   ErrorType;
    char       ErrorMessage[KXMERROR_MSG_LEN+1];
} KXMSUBERROR_1;

#define KXMSUBERROR_DEFAULT_1   KXMSIG_SUBERROR,\
                                KXMVER_SUBERROR_1,\
                                0, 0, 0, 0, 0, ""

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    KXMULONG   ReturnCode;
    KXMULONG   OpType;
    KXMULONG   Reason;
    KXMULONG   Position;
    KXMULONG   ErrorType;
    char       ErrorMessage[KXMERROR_MSG_LEN+1];
    char       Location[KXMLOCATION_LEN+1];
} KXMSUBERROR;

#define KXMSUBERROR_DEFAULT     KXMSIG_SUBERROR,\
                                KXMVER_SUBERROR_CURRENT,\
                                0, 0, 0, 0, 0, "", ""

typedef struct
{
    KXMULONG      Signature;
    KXMULONG      Version;
    KXMSUBERROR_1 Primary;
    KXMSUBERROR_1 Secondary;
} KXMERROR_1;

typedef struct
{
    KXMULONG    Signature;
    KXMULONG    Version;
    KXMSUBERROR Primary;
    KXMSUBERROR Secondary;
} KXMERROR;

#define KXMERROR_DEFAULT_1      KXMSIG_ERROR,\
                                KXMVER_ERROR_1,\
                                { KXMSUBERROR_DEFAULT_1 },\
                                { KXMSUBERROR_DEFAULT_1 }

#define KXMERROR_DEFAULT        KXMSIG_ERROR,\
                                KXMVER_ERROR_CURRENT,\
                                { KXMSUBERROR_DEFAULT },\
                                { KXMSUBERROR_DEFAULT }

/********************************************************************/
/* Time zone structure                                              */
/********************************************************************/

#define KXMSIG_TIMEZONE         KXMSIG('T','Z')
#define KXMVER_TIMEZONE_1       1
#define KXMVER_TIMEZONE_CURRENT 1

typedef struct
{
    KXMULONG	Signature;
    KXMULONG	Version;
    KXMULONG	Type;
    KXMULONG	Hours;
    KXMULONG	Minutes;
} KXMTIMEZONE;

#define KXMTIMEZONE_TYPE_UNKNOWN    0
#define KXMTIMEZONE_TYPE_UTC        1
#define KXMTIMEZONE_TYPE_POS        2
#define KXMTIMEZONE_TYPE_NEG        3

#define KXMTIMEZONE_DEFAULT KXMSIG_TIMEZONE,\
                            KXMVER_TIMEZONE_CURRENT,\
                            KXMTIMEZONE_TYPE_UNKNOWN,\
                            0, 0

/********************************************************************/
/* Date and time structure                                          */
/********************************************************************/

#define KXMSIG_DATE             KXMSIG('D','T')
#define KXMVER_DATE_1           1
#define KXMVER_DATE_2           2
#define KXMVER_DATE_CURRENT     2

#define KXMDATE_STATUS_NONE         0
#define KXMDATE_STATUS_DATE         1
#define KXMDATE_STATUS_TIME         2
#define KXMDATE_STATUS_DATETIME     3
#define KXMDATE_STATUS_GYEAR        4
#define KXMDATE_STATUS_GYEARMONTH   5
#define KXMDATE_STATUS_GMONTHDAY    6
#define KXMDATE_STATUS_GDAY         7
#define KXMDATE_STATUS_GMONTH       8

typedef struct
{
    KXMULONG        Signature;
    KXMULONG        Version;
    KXMULONG        Status;
    KXMULONG        Year;
    KXMULONG        Month;
    KXMULONG        Day;
    KXMULONG        Hour;
    KXMULONG        Min;
    KXMULONG        Sec;
    KXMULONG        Msec;
} KXMDATE_1;

typedef struct
{
    KXMULONG        Signature;
    KXMULONG        Version;
    KXMULONG        Status;
    KXMULONG        Year;
    KXMULONG        Month;
    KXMULONG        Day;
    KXMULONG        Hour;
    KXMULONG        Min;
    KXMULONG        Sec;
    KXMULONG        Msec;
    KXMTIMEZONE     TimeZone;
} KXMDATE;

#define KXMDATE_DEFAULT_1       KXMSIG_DATE,\
                                KXMVER_DATE_1,\
                                KXMDATE_STATUS_NONE,\
                                0, 0, 0,  0, 0, 0, 0

#define KXMDATE_DEFAULT         KXMSIG_DATE,\
                                KXMVER_DATE_CURRENT,\
                                KXMDATE_STATUS_NONE,\
                                0, 0, 0,  0, 0, 0, 0,\
                                { KXMTIMEZONE_DEFAULT }

/********************************************************************/
/* KXMDURATION structure                                            */
/********************************************************************/

typedef struct
{
    KXMULONG	Signature;
    KXMULONG	Version;
    KXMBOOL     Negative;
    KXMULONG	Years;
    KXMULONG	Months;
    KXMULONG	Days;
    KXMULONG	Hours;
    KXMULONG	Minutes;
    KXMULONG	Seconds;
    KXMULONG    Msec;
} KXMDURATION;

#define KXMSIG_DURATION             KXMSIG('D','U')
#define KXMVER_DURATION_1           1
#define KXMVER_DURATION_CURRENT     1

#define KXMDURATION_DEFAULT         KXMSIG_DURATION,\
                                    KXMVER_DURATION_CURRENT,\
                                    FALSE,  0, 0, 0,  0, 0, 0, 0

/********************************************************************/
/* KXMQNAME structure                                               */
/********************************************************************/

typedef struct
{
    KXMULONG	    Signature;
    KXMULONG	    Version;
    const KXMCHAR*	NsUri;
    KXMLONG	        NsUriLength;
    const KXMCHAR*	NsPrefix;
    KXMLONG	        NsPrefixLength;
    const KXMCHAR*	LocalPart;
    KXMLONG	        LocalPartLength;
    const KXMCHAR*	Name;
    KXMLONG	        NameLength;
} KXMQNAME;

#define KXMSIG_QNAME             KXMSIG('Q','N')
#define KXMVER_QNAME_1           1
#define KXMVER_QNAME_CURRENT     1

#define KXMQNAME_DEFAULT         KXMSIG_QNAME,\
                                 KXMVER_QNAME_CURRENT,\
                                 NULL, -1, NULL, -1,\
                                 NULL, -1, NULL, -1

/********************************************************************/
/* Array value                                                      */
/********************************************************************/
#define KXMVER_ARRAYVALUE_1         1
#define KXMVER_ARRAYVALUE_CURRENT   1
#define KXMSIG_ARRAYVALUE       KXMSIG('A','V')

#define KXMVALUETYPE_LONG       1
#define KXMVALUETYPE_ULONG      2

#ifndef KXM_NO_IEEE_FLOAT
#define KXMVALUETYPE_DOUBLE     3
#endif

#define KXMVALUETYPE_BINARY     4
#define KXMVALUETYPE_STRING     5
#define KXMVALUETYPE_DATE       6

#ifndef KXM_NO_INT64
#define KXMVALUETYPE_INT64      7
#define KXMVALUETYPE_UINT64     8
#endif

#define KXMVALUETYPE_INT32      9
#define KXMVALUETYPE_UINT32     10
#define KXMVALUETYPE_BOOL       11
#define KXMVALUETYPE_QNAME      12
#define KXMVALUETYPE_DURATION   13
#define KXMVALUETYPE_USTRING    14

typedef union
{
    long*               LongValue;
    unsigned long*      ULongValue;
#ifndef KXM_NO_IEEE_FLOAT
    double*             DoubleValue;
#endif
    const void**        BinaryValue;
    const KXMCHAR**     StringValue;
    KXMDATE*            DateValue;
#ifndef KXM_NO_INT64
    KXMINT64*           Int64Value;
    KXMUINT64*          UInt64Value;
#endif
    KXMINT32*           Int32Value;
    KXMUINT32*          UInt32Value;
    KXMBOOL*            BoolValue;
    KXMQNAME*           QNameValue;
    KXMDURATION*        DurationValue;
    const KXMUCCHAR**   UStringValue;
} KXMVALUE;

typedef struct
{
    KXMULONG   Signature;
    KXMULONG   Version;
    KXMULONG   ValueType;
    KXMULONG   Index;
    KXMULONG   Count;
    KXMVALUE   Value;
    KXMULONG*  ValueLength;
    KXMBOOL*   IsEmpty;
} KXMARRAYVALUE;

#define KXMARRAYVALUE_DEFAULT   KXMSIG_ARRAYVALUE,\
                                KXMVER_ARRAYVALUE_CURRENT,\
                                0, 0, 0, NULL, NULL, NULL

/********************************************************************/
/* Field information                                                */
/********************************************************************/

#define KXMFIELDTYPE_NONE       0
#define KXMFIELDTYPE_STRING     1
#define KXMFIELDTYPE_NUMBER     2
#define KXMFIELDTYPE_BINARY     3
#define KXMFIELDTYPE_XMLELEM    4
#define KXMFIELDTYPE_XMLATTR    5
#define KXMFIELDTYPE_DATE       6
#define KXMFIELDTYPE_GROUP      7
#define KXMFIELDTYPE_BOOL       8
#define KXMFIELDTYPE_QNAME      9
#define KXMFIELDTYPE_DURATION   10

#define KXMDATETYPE_NONE        0
#define KXMDATETYPE_DATE        1
#define KXMDATETYPE_TIME        2
#define KXMDATETYPE_DATETIME    3
#define KXMDATETYPE_GYEAR       4
#define KXMDATETYPE_GYEARMONTH  5
#define KXMDATETYPE_GMONTHDAY   6
#define KXMDATETYPE_GDAY        7
#define KXMDATETYPE_GMONTH      8

#define KXMNUMTYPE_NONE         0
#define KXMNUMTYPE_UINT8        100
#define KXMNUMTYPE_INT8         101
#define KXMNUMTYPE_UINT16       102
#define KXMNUMTYPE_INT16        103
#define KXMNUMTYPE_UINT32       104
#define KXMNUMTYPE_INT32        105
#define KXMNUMTYPE_DECIMAL      106
#define KXMNUMTYPE_INTEGER      107
#define KXMNUMTYPE_DOUBLE       108
#define KXMNUMTYPE_UINT64       109
#define KXMNUMTYPE_INT64        110
#define KXMNUMTYPE_FLOAT        111

#define KXMFIELDFLAG_IS_ROOT            1
#define KXMFIELDFLAG_IS_ARRAY           2
#define KXMFIELDFLAG_IS_GROUP           4
#define KXMFIELDFLAG_IS_DEFAULT_VALUE   8
#define KXMFIELDFLAG_IS_UNION           16
#define KXMFIELDFLAG_IS_XMLELEM         32
#define KXMFIELDFLAG_IS_XMLATTR         64
#define KXMFIELDFLAG_IS_XMLDOC          128

#define KXMSIG_FIELDINFO            KXMSIG('F','I')
#define KXMVER_FIELDINFO_1          1
#define KXMVER_FIELDINFO_2          2
#define KXMVER_FIELDINFO_3          3
#define KXMVER_FIELDINFO_CURRENT    KXMVER_FIELDINFO_3

typedef struct
{
    KXMULONG       Signature;
    KXMULONG       Version;
    KXMFIELDNAMESZ FieldName;
    KXMULONG       FieldType;
    KXMULONG       NumericType;
    KXMULONG       ValMinLength;
    KXMULONG       ValMaxLength;
    KXMULONG       ArrayMinLength;
    KXMULONG       ArrayMaxLength;
    KXMFIELDNAMESZ FormatName;
    KXMULONG       FieldCount;
    KXMULONG       Reserved2;
    void*          Reserved1;
} KXMFIELDINFO_1;

typedef struct
{
    KXMULONG       Signature;
    KXMULONG       Version;
    KXMFIELDNAMESZ FieldName;
    KXMULONG       FieldType;
    KXMULONG       NumericType;
    KXMULONG       ValMinLength;
    KXMULONG       ValMaxLength;
    KXMULONG       ArrayMinLength;
    KXMULONG       ArrayMaxLength;
    KXMFIELDNAMESZ FormatName;
    KXMULONG       FieldCount;
    KXMULONG       DateType;
    void*          Reserved1;
    KXMULONG       Flags;
} KXMFIELDINFO_2;

typedef struct
{
    KXMULONG        Signature;
    KXMULONG        Version;
    KXMFIELDNAMESZ  FieldName;
    KXMULONG        FieldType;
    KXMULONG        NumericType;
    KXMULONG        ValMinLength;
    KXMULONG        ValMaxLength;
    KXMULONG        ArrayMinLength;
    KXMULONG        ArrayMaxLength;
    KXMFIELDNAMESZ  FormatName;
    KXMULONG        FieldCount;
    KXMULONG        DateType;
    const KXMCHAR*  LocalPart;
    KXMULONG        Flags;
    KXMULONG        LocalPartLength;
    const KXMCHAR*  NsPrefix;
    KXMULONG        NsPrefixLength;
    const KXMCHAR*  NsUri;
    KXMULONG        NsUriLength;
} KXMFIELDINFO;

#define KXMFIELDINFO_DEFAULT_1  KXMSIG_FIELDINFO,\
                                KXMVER_FIELDINFO_1,\
                                "", 0, 0, 0, 0, 0, 0, "", 0, 0, NULL

#define KXMFIELDINFO_DEFAULT_2  KXMSIG_FIELDINFO,\
                                KXMVER_FIELDINFO_CURRENT,\
                                "", 0, 0, 0, 0, 0, 0, "", 0, 0, NULL, 0

#define KXMFIELDINFO_DEFAULT    KXMSIG_FIELDINFO,\
                                KXMVER_FIELDINFO_CURRENT,\
                                "", 0, 0, 0, 0, 0, 0, "", 0, 0, "", 0,\
                                0, "", 0, "", 0

/********************************************************************/
/* Message information                                              */
/********************************************************************/

#define KXMSIG_MSGINFO            KXMSIG('M','I')
#define KXMVER_MSGINFO_1          1
#define KXMVER_MSGINFO_CURRENT    1

typedef struct
{
    KXMULONG        Signature;
    KXMULONG        Version;
    KXMFORMATNAMESZ FormatName;
    KXMULONG        FieldCount;
    const KXMCHAR*  XmlDocName;
    KXMULONG        XmlDocNameLength;
} KXMMSGINFO;

#define KXMMSGINFO_DEFAULT      KXMSIG_MSGINFO,\
                                KXMVER_MSGINFO_CURRENT,\
                                "", 0, "", 0,

#ifndef    KXM_NO_SIGNATURES

/********************************************************************/
/* Functions                                                        */
/********************************************************************/

/********************************************************************/
/* Error functions                                                  */
/********************************************************************/
KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetLastError(
    KXMERROR* error
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetLastError(
    const KXMERROR* error
);

/********************************************************************/
/* Initialization functions                                         */
/********************************************************************/

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmInitialize(
    const KXMINITOPT* options,
    void* reserved
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmTerminate(
    const void* reserved1,
    void* reserved2 );

/********************************************************************/
/* Message functions                                                */
/********************************************************************/

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmCreateMessage(
    const KXMCHAR* formatName,
    const KXMMSGOPT* options,
    void* reserved,
    HKXMMSG* message
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmParseMessage(
    const KXMCHAR* formatName,
    const void* data,
    KXMULONG dataLength,
    const KXMMSGOPT* options,
    void* reserved,
    HKXMMSG* message
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmExtractMessageData(
    const HKXMMSG message,
    const KXMEXTOPT* options,
    KXMBUFFER* buffer,
    void* reserved
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmCloseHandle(
    HKXMFIELD* field,
    const void* reserved1,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmFreeBuffer(
    KXMBUFFER* buffer,
    const void* reserved1,
    void* reserved2
);

/********************************************************************/
/* Transport API dependent functions                                */
/********************************************************************/

#ifdef KXC_INCLUDED

KXMEXPORT(KXMRCHFAPI,KXMBOOL) kxmParseRomaMessage(
    const KXCMSG* romaMessage,
    const KXMMSGOPT* options,
    void* reserved,
    HKXMMSG* message
);

KXMEXPORT(KXMRCHFAPI,KXMBOOL) kxmExtractRomaMessage(
    HKXC hKXC,
    const HKXMMSG message,
    KXCMSG* romaMessage,
    const KXMEXTOPT* opt,
    void* reserved2
);

#endif

/********************************************************************/
/* Field functions                                                  */
/********************************************************************/

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmIsEmpty(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMBOOL* isEmpty,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetLong(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    long* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetULong(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    unsigned long* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetInt32(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMINT32* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetBool(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMBOOL* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetUInt32(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMUINT32* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#ifndef KXM_NO_INT64

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetInt64(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMINT64* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetUInt64(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMUINT64* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#endif

#ifndef KXM_NO_IEEE_FLOAT

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetDouble(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    double* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#endif

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetString(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMCHAR** value,
    KXMULONG* valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetUString(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMUCCHAR** value,
    KXMULONG* valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetDate(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMDATE* date,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetBinary(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    const void** value,
    KXMULONG* valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetQName(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMQNAME* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetDuration(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMDURATION* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetLong(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    long value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetULong(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    unsigned long value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetInt32(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMINT32 value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetUInt32(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMUINT32 value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetBool(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMBOOL value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#ifndef KXM_NO_INT64

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetInt64(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMINT64 value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetUInt64(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMUINT64 value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#endif

#ifndef KXM_NO_IEEE_FLOAT

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetDouble(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    double value,
    const KXMFLDOPT* opt,
    void* reserved2
);

#endif

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetString(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMCHAR* value,
    KXMLONG valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetUString(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMUCCHAR* value,
    KXMLONG valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetDate(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMDATE* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetBinary(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const void* value,
    KXMULONG valueLength,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetQName(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMQNAME* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetDuration(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMDURATION* value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetField(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    HKXMFIELD value,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmDeleteField(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetField(
    const HKXMFIELD message,
    const KXMCHAR* fieldName,
    KXMLONG indexInArray,
    const KXMFLDOPT* opt,
    void* reserved2,
    HKXMFIELD* field
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmDeclareNs(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMFLDOPT* opt,
    const KXMCHAR* nsPrefix,
    KXMLONG prefixLength,
    const KXMCHAR* nsURI,
    KXMLONG uriLength
);

/********************************************************************/
/* Array functions                                                  */
/********************************************************************/

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetArrayLength(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMULONG* length,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetArrayLength(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMULONG length,
    const KXMFLDOPT* opt,
    void* reserved2
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmSetArrayValue(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMFLDOPT* opt,
    const KXMARRAYVALUE* value,
    void* reserved );

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetArrayValue(
    HKXMFIELD field,
    const KXMCHAR* fieldName,
    const KXMFLDOPT* opt,
    KXMARRAYVALUE* value,
    void* reserved );

/********************************************************************/
/* Advanced functions                                               */
/********************************************************************/

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetFieldInfo(
    const HKXMFIELD field,
    const KXMCHAR* fieldName,
    KXMLONG fieldIndex,
    const KXMFLDOPT* opt,
    KXMFIELDINFO* fieldInfo
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmGetMessageInfo(
    HKXMMSG message,
    const void* reserved,
    KXMMSGINFO* info
);

KXMEXPORT(KXMCHFAPI,KXMBOOL) kxmCopyMessage(
    HKXMMSG message,
    HKXMMSG* copy
);

#endif  /* KXM_NO_SIGNATURES */

/********************************************************************/
/* Macros                                                           */
/********************************************************************/

#define kxmDeleteMessage( hMsg, reserved1, reserved2 )\
            kxmCloseHandle(hMsg,reserved1,reserved2)

#define KXM_SetBuffer( pKxmBuffer, pBuffer, bufferLen )\
            if ( (pKxmBuffer)->Owner == KXMBUFOWN_KXM )\
            {\
                kxmFreeBuffer(pKxmBuffer,NULL,NULL);\
            }\
            (pKxmBuffer)->Buffer = pBuffer;\
            (pKxmBuffer)->BufferLen = bufferLen;\
            (pKxmBuffer)->DataLen = bufferLen;\
            (pKxmBuffer)->Owner = KXMBUFOWN_USER

#define KXM_SetLongArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_LONG;\
            (pKxmArrayVal)->Value.LongValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetULongArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_ULONG;\
            (pKxmArrayVal)->Value.ULongValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetInt32ArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_INT32;\
            (pKxmArrayVal)->Value.Int32Value = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetUInt32ArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_UINT32;\
            (pKxmArrayVal)->Value.UInt32Value = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#ifndef KXM_NO_IEEE_FLOAT

#define KXM_SetDoubleArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_DOUBLE;\
            (pKxmArrayVal)->Value.DoubleValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#endif

#define KXM_SetBinaryArrayVal( pKxmArrayVal, pValue, pValueLen, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_BINARY;\
            (pKxmArrayVal)->Value.BinaryValue = pValue;\
            (pKxmArrayVal)->ValueLength = pValueLen;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetStringArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_STRING;\
            (pKxmArrayVal)->Value.StringValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetUStringArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_USTRING;\
            (pKxmArrayVal)->Value.UStringValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#ifndef KXM_NO_INT64

#define KXM_SetInt64ArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_INT64;\
            (pKxmArrayVal)->Value.Int64Value = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetUInt64ArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_UINT64;\
            (pKxmArrayVal)->Value.UInt64Value = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#endif

#define KXM_SetDateArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_DATE;\
            (pKxmArrayVal)->Value.DateValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetBoolArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_BOOL;\
            (pKxmArrayVal)->Value.BoolValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetDurationArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_DURATION;\
            (pKxmArrayVal)->Value.DurationValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXM_SetQNameArrayVal( pKxmArrayVal, pValue, count )\
            (pKxmArrayVal)->ValueType = KXMVALUETYPE_QNAME;\
            (pKxmArrayVal)->Value.QNameValue = pValue;\
            (pKxmArrayVal)->ValueLength = NULL;\
            (pKxmArrayVal)->Count = count;\
            (pKxmArrayVal)->Index = 0

#define KXMGETFIELD_PARENT     ((KXMULONG)-3)
#define KXMGETFIELD_DEFAULT    ((KXMULONG)-2)
#define KXMGETFIELD_NOTINDEX   ((KXMULONG)-1)

#define KXM_ZSTR             ((KXMULONG)-1)



/********************************************************************/
/* Enable C++ support                                               */
/********************************************************************/
#if defined(__cplusplus)
  }
#endif

/********************************************************************/
/* End protection against multiple includes                         */
/********************************************************************/
#endif

/********************************************************************/
/* End of module KXMCHF.H                                           */
/********************************************************************/
